require "faye/websocket"
require "openssl"

module Ginbee
  module WebSocket
    class Client < Faye::WebSocket::Client
      private

      def on_connect(stream)
        if @secure
          cert_store = OpenSSL::X509::Store.new
          ca_file = @socket_tls[:ca_file]
          cert_store.add_file(ca_file)
          hostname = @socket_tls[:sni_hostname] || URI.parse(@url).host

          stream.define_singleton_method(:ssl_verify_peer) do |cert_str|
            cert = OpenSSL::X509::Certificate.new(cert_str)
            cert_store.verify(cert)
          end

          stream.define_singleton_method(:ssl_handshake_completed) do
            cert_str = get_peer_cert
            cert = OpenSSL::X509::Certificate.new(cert_str)
            unless OpenSSL::SSL.verify_certificate_identity(cert, hostname)
              unbind("hostname does not match the server certificate")
            end
          end
        end
        super(stream)
      end
    end
  end
end
