# coding:utf-8
require 'net/https'
require 'open3'
require 'fileutils'

module PushExchanger
  module Util

    module_function

    def print(filename,printer,num,print_command=nil)
      if RUBY_PLATFORM.downcase =~ /mswin(?!ce)|mingw|cygwin|bccwin/
        win_print(print_command,filename,printer,num)
      else
        linux_print(filename,printer,num)
      end
    end

    def win_print(print_command,filename,printer,num)
      unless print_command
        @logger.info(":print_command nothing in config")
        return
      end
      cmd = print_command.dup
      cmd.sub!(/%FILE%/,filename)
      cmd.sub!(/%PRINTER%/,printer)
      cmd.sub!(/%NUM%/,num.to_s)
      @logger.info(cmd)
      Open3.popen3(cmd) do |i, o, e, w|
        i.close
        o.each do |line| @logger.debug(line) end
        e.each do |line| @logger.debug(line) end
      end
    end

    def linux_print(filename,printer,num)
      cmd = "lpr -P #{printer} -# #{num} #{filename}"
      @logger.info(cmd)
      Open3.popen3(cmd) do |i, o, e, w|
        i.close
        o.each do |line| @logger.debug(line) end
        e.each do |line| @logger.debug(line) end
      end
    end

    def http_request(conf,path,data=nil,content_type="application/json")
      Net::HTTP.version_1_2
      https = Net::HTTP.new(conf[:api_server],conf[:api_port])
      if conf[:use_ssl]
        https.use_ssl = true
        https.ca_file = conf[:ca_cert]
        https.cert = OpenSSL::X509::Certificate.new(File.read(conf[:cert]))
        https.key = OpenSSL::PKey::RSA.new(File.read(conf[:cert_key]),conf[:passphrase])
        https.verify_mode = OpenSSL::SSL::VERIFY_PEER
        https.verify_depth = 5
      end

      https.start do
        req = nil
        if data
          req = Net::HTTP::Post.new(path)
          req.content_length = data.size
          req.content_type = content_type
          req.body = data
        else
          req = Net::HTTP::Get.new(path)
        end
        req.basic_auth(conf[:api_user],conf[:api_key])
        res = https.request(req)
        res
      end
    end

    def http_post(conf,path,data,content_type="application/json")
      http_request(conf,path,data,content_type)
    end

    def http_get(conf,path)
      http_request(conf,path)
    end

    def file_logger(dir,fname,limit)
       FileUtils.mkdir_p(dir)
       yield
       if limit > 0
        list = Dir.glob(File.join(dir,"*")).sort do |a,b|
          File.ctime(a) <=> File.ctime(b)
        end
        if list.size > limit
          FileUtils.rm_f(list[0,list.size - limit])
        end
      end
    end

  end
end
