# coding:utf-8
require 'pp'
require 'fileutils'
require 'securerandom'
require 'net/https'
require 'socket'
require 'thread'
require 'logger'
require 'yaml'
require 'json'
require 'socket'
require 'rexml/document'

require_relative "claim-util"

class Claim
  # accept

  def accept(data)
    @conf[:logger].info("claim accept start")

    xml = nil
    body = data["body"]
    begin
      xml = make_front_xml(body)
      xml_log("accept",xml)
    rescue Exception => ex
      STDERR.puts ex.backtrace
      @conf[:logger].info ex
      raise ex
    end

    return unless xml
    @claim_conf[:servers].each_with_index do |r,i|
      next unless r[:event].include?("accept")
      @conf[:logger].info "send #{r[:name]}"
      res = ERR
      begin
        res = send_claim(r,xml)
        @conf[:logger].info "recv #{h_code(res)}"
      rescue Exception => ex
        @conf[:logger].info ex
        @conf[:logger].info ex.backtrace[0,2]
      end
      if i == 0
        send_accept_result(res,body)
      end
    end
    @conf[:logger].info("claim accept end")
  end

  def remove_accept_const(doc)
    doc.get_elements("/Mml/MmlBody").each do |body|
      list = []
      body.get_elements("MmlModuleItem").each do |item|
        list << item if item.elements.size <= 1
      end
      list.each do |item|
        body.delete_element(item)
      end
    end
  end
  
  def embed_front_xml(jsonstr)
    # parse def file
    var = {}
    deffile = File.join(File.dirname(__FILE__),"template","orca_layout_front.def")
    File.read(deffile).split("\n").each do |l|
      r = l.split(",")
      if r[1] == "const"
        var[r[0]] = {"type" => "const", "const" => r[2] }
      elsif r[5]
        var[r[0]] = {"type" => "attr", "attr" => r[5] }
      else
        var[r[0]] = {"type" => "normal"}
      end
    end
  
    # read template
    tmpl = File.join(File.dirname(__FILE__),"template","orca_base_front.xml")
    xml = File.read(tmpl)
  
    # convert json to value list
    json = JSON.parse(jsonstr)
    json = json["claim_frontv2res"]
    unless json["Api_Result"] == "00"
      raise "claim api error:#{json["Api_Result_Message"]}"
    end
    json_list = {}
    flatten_json(json_list,"",json)
  
    data = Hash.new("")
    json_list.each do |k,v|
      if %r|^HealthInsurance_Information\[(\d+)\]\.(.*)| =~ k
        i = $1.to_i + 1
        kk = $2
        if %r|^PublicInsurance_Information\[(\d)\]\.(.*)| =~ kk
          j = $1.to_i + 1
          kkk = $2
          kkk.sub!("ii","%02d"%i).sub!("jj","%d"%j)
          data["%#{kkk}%"] = v
        else
          kk.sub!("ii","%02d"%i)
          data["%#{kk}%"] = v
        end
      else
        data["%#{k}%"] = v
      end
    end
  
    # convert
    var.each do |k,v|
      sub = ""
      case v["type"]
      when "const"
        sub = v["const"]
      when "attr"
        if data[k] && !data[k].empty?
          sub = %| #{v["attr"]}="#{data[k]}"|
        else
          sub = ""
        end
      else
        sub = data[k]
      end
      xml.sub!(k,sub)
    end
  
    doc = REXML::Document.new(xml,{:compress_whitespace => :all})
    canonicalize(doc.root)
    remove_accept_const(doc)
    xml = doc2xml(doc)
  end
  
  def make_front_xml(msg)
    req = {"claim_frontv2req" => msg }
    h = req["claim_frontv2req"]
    h["Request_Number"] = "01"
    h["Send_Status"] = "00"
    res = http_post(@conf,"/api01rv2/claimfrontv2",req.to_json)
    unless res.code == "200"
      raise "api error /api01rv2/claimfrontv2 #{res.code}"
    end
    xml = embed_front_xml(res.body)
  end
  
  def send_accept_result(res,msg)
    req = {"claim_frontv2req" => msg }
    h = req["claim_frontv2req"]
    h["Request_Number"] = "02"
    case res
    when ACK
      h["Send_Status"] = "00"
    when NAK
      h["Send_Status"] = "02"
    else
      h["Send_Status"] = "01"
    end
    res = http_post(@conf,"/api01rv2/claimfrontv2",req.to_json)
    unless res.code == "200"
      raise "send_accept_result api error /api01rv2/claimfrontv2 #{res.code}"
    end
  end
end
