require 'pp'
require 'fileutils'
require 'securerandom'
require 'net/https'
require 'socket'
require 'thread'
require 'logger'
require 'yaml'
require 'json'
require 'socket'
require 'rexml/document'

require 'push_exchanger'

class Claim
  include PushExchanger::Util

  EOT = 0x04.chr
  ACK = 0x06.chr
  NAK = 0x15.chr
  ERR = 0x16.chr

  def xml_log(type,xml)
    dir   = @claim_conf[:xml_log_dir]
    limit = @claim_conf[:xml_log_limit] || 100
    if dir
      fname = Time.now.strftime("#{type}-%Y%m%d%H%M%S-#{SecureRandom.uuid}.xml")
      fname = File.join(dir,fname)

      file_logger(dir,fname,limit) do
        File.open(fname,"wb") do |io| io.print xml end
      end
    end
  end

  def flatten_json(out,k,v)
    case v.class.to_s
    when "String","Fixnum","Float"
      out[k] = v
    when "Array"
      v.each_with_index do |vv,i|
        flatten_json(out,"#{k}[#{i}]",vv)
      end
    when "Hash"
      v.each do |kk,vv|
        if k.empty?
          flatten_json(out,"#{kk}",vv)
        else
          flatten_json(out,"#{k}.#{kk}",vv)
        end
      end
    end
  end

  def canonicalize(elem)
    if elem.has_elements?
      children = []
      elem.each do |c|
        unless c.node_type == :element
          children << c
        end
      end
      children.each do |c|
        elem.delete(c)
      end
  
      have_data = false
      children = []
      elem.each_element do |c|
        if canonicalize(c)
          have_data = true
        else
          children << c
        end
      end
      children.each do |c|
        elem.delete_element(c)
      end
      return have_data
    else
      text = elem.text
      if text == nil || text.empty? || /\A\s*\z/ =~ text
        return false
      else
        text = text.strip
        elem.text = text
      end
      return true
    end
    puts "do not reach"
    exit 1
  end

  def doc2xml(doc,encoding='UTF-8')
    doc.xml_decl.encoding = encoding
    sio = StringIO.new('',"wb")
    sio.set_encoding(encoding)
    doc.write(sio,-1,false)
    sio.close
    sio.string
  end
  
  def xml_encoding(xml,encoding)
    if encoding == "UTF-8"
      return xml
    else
      doc = REXML::Document.new(xml,{:compress_whitespace => :all})
      return doc2xml(doc,encoding)
    end
  end

  def h_code(c)
    case c
    when EOT
      return "EOT"
    when ACK
      return "ACK"
    when NAK
      return "NAK"
    else
      return "ERR"
    end
  end

  def send_claim(r,xml)
    xml = xml_encoding(xml,r[:encoding])
    s = TCPSocket.open(r[:host],r[:port])
    s.write(xml)
    s.write(EOT)
    res = s.recv(1)
    s.close
    res
  end
end

