# coding : utf-8

require 'thinreports'
require_relative '../lib/orca-string'
require_relative '../lib/orca-wareki'

class A32001h01

  FIRST_PAGE_DISEASE_ROWS = 12
  MIDDLE_PAGE_DISEASE_ROWS = 22

  # call from print001.rb
  def export(fname,report_data,opt)
    report = ThinReports::Report.new
    form_dir = File.dirname(File.expand_path(__FILE__))
    tmpl_first = File.join(form_dir, 'a32001h01.tlf')
    tmpl_middle = File.join(form_dir, 'h011.tlf')
    report.use_layout tmpl_first, id: :first
    report.use_layout tmpl_middle, id: :middle

    report_data['Forms'].each do |hash|

      data = hash['data']
      if data['Order_Class'] == '2'

        report.start_new_page layout: :middle do |page|
          patient = data['Patient']
          page.item("Patient_ID").value(patient['ID'])
          page.item("Name").value(patient['Name'])
        end

        next
      end

      if data['Disease_Information'].nil?
        data['Disease_Information'] = []
      else
        data['Disease_Information'].delete_if{|item| item['Name'].to_s.empty?}
      end

      edit_first_page(report, data)

      start_row = FIRST_PAGE_DISEASE_ROWS
      while data['Disease_Information'].size  > start_row
        edit_middle_page(report, data, start_row)
        start_row += MIDDLE_PAGE_DISEASE_ROWS
      end
    end

    report.generate(filename: fname)
  end

  def edit_disease_row(disease)
    row = Hash.new
    row['Name'] = disease['Name']
    ymd = Orca::Wareki.new(disease['StartDate'])
    row['StartDate'] = "%2d.%2d.%2d" % [ymd.year_w, ymd.month, ymd.day]
    if ! disease['EndDate'].to_s.empty?
      ymd = Orca::Wareki.new(disease['EndDate'])
      row['EndDate'] = "%2d.%2d.%2d" % [ymd.year_w, ymd.month, ymd.day]
    else
      row['EndDate'] = "  .  ."
    end

    case disease['OutCome_Class']
    when '1'
      row['Cure'] = '○'
    when '2'
      row['Death'] = '○'
    when '3'
      row['Discontinued'] = '○'
    end

    return row
  end

  def edit_first_page(report, data)

    report.start_new_page layout: :first do |page|

      patient = data['Patient'] || {}
      hospital = data['Hospital'] || {}
      insurance = data['Insurance_Combination_Information'] || {}
      h_insurance = insurance['HealthInsurance_Information'] || {}
      a_insurance = insurance['Accident_Insurance_Information'] || {}

      page.item('Patient_ID').value(patient['ID'])
      page.item('Hospital_Name').value(hospital['Name'])
      page.item('KanaName').value(patient['KanaName'])
      page.item('Name').value(patient['Name'])
      ymd=Orca::Wareki.new(patient['BirthDate'])
      page.item('BirthDate_G').value(ymd.gengoN)
      page.item('BirthDate_Y').value('%2d' % ymd.year_w)
      page.item('BirthDate_M').value('%2d' % ymd.month)
      page.item('BirthDate_D').value('%2d' % ymd.day)
      page.item('Age').value(ymd.age)

      case patient['Sex']
      when '1'
        page.item('Man').value('○')
      when '2'
        page.item('Woman').value('○')
      end

      home_address = patient['Home_Address'] || {}
      page.item('Address').value([home_address['Address']].join)
      page.item('PhoneNumber').value(home_address['PhoneNumber'])
      page.item('Job').value(patient['Job'])
      page.item('Relationship').value(patient['Relationship'])

      page.item('Memo').value([patient['Taboo'], patient['Allergy'], patient['Infection']].join('　'))

      case insurance['InsuranceProvider_Class']
      when '975'
        page.item('InsuranceProvider_Number').value(insurance['Insurance_Name'])
      when '971', '973'
        page.item('InsuranceProvider_Number').value([h_insurance['HealthInsuredPerson_Symbol'] ,
          "・　" , h_insurance['HealthInsuredPerson_Number']].join)
        page.item('SymbolAndNumber').value(a_insurance['Accident_Insurance_Number'])
      else
        h_insurance['InsuranceProvider_Number'].to_s.zenkaku.reverse.split(//)[0,8].each.with_index(1) do |num, i|
          page.item('InsuranceProvider_Number_%d' % i).value(num)
        end
        if ! [h_insurance['HealthInsuredPerson_Symbol'], h_insurance['HealthInsuredPerson_Number']].compact.empty?
          if insurance['InsuranceProvider_Class'] == '039'
            page.item('SymbolAndNumber').value(h_insurance['HealthInsuredPerson_Number'])
          else
            delimiter = "・　"
            if ['060', '067'].include?(insurance['InsuranceProvider_Class'])
              delimiter = "　"
            end
            page.item('SymbolAndNumber').value([h_insurance['HealthInsuredPerson_Symbol'],
              delimiter , h_insurance['HealthInsuredPerson_Number']].join)
          end
        end
      end

      if ! ['9999' , '' ].include?(h_insurance['Certificate_ExpiredDate'].to_s[0,4].strip)
        ymd=Orca::Wareki.new(h_insurance['Certificate_ExpiredDate'])
        page.item('Certificate_ExpiredDate').value(ymd.wareki_n)
      else
        ymd=Orca::Wareki.new
        page.item('Certificate_ExpiredDate').value("#{ymd.gengoN}　　年　　月　　日")
      end

      page.item('HealthInsuredPerson_Name').value(h_insurance['HealthInsuredPerson_Name'])

      if ! ['9999' , '' ].include?(h_insurance['Certificate_GetDate'].to_s[0,4].strip)
        ymd=Orca::Wareki.new(h_insurance['Certificate_GetDate'])
        page.item('Certificate_GetDate').value(ymd.wareki_n)
      else
        ymd=Orca::Wareki.new
        page.item('Certificate_GetDate').value("#{ymd.gengoN}　　年　　月　　日")
      end

      office_address = patient['Office_Address'] || {}
      page.item('OfficeAddress').value([office_address['Address']].join)
      page.item('OfficePhoneNumber').value(office_address['PhoneNumber'])

      page.item('InsuranceProvider_Address').value([h_insurance['InsuranceProvider_Address']].join)
      page.item('InsuranceProvider_PhoneNumber').value(h_insurance['InsuranceProvider_PhoneNumber'])
      page.item('InsuranceProvider_Name').value(h_insurance['InsuranceProvider_Name'])
      Array(insurance['PublicInsurance_Information'])[0,3].each.with_index(1) do |p_insurance , i|
        p_insurance['PublicInsurer_Number'].to_s.zenkaku.split(//)[0, 8].each_with_index do |num, j|
          page.item('PublicInsurer_Number%d_%d' % [i, 8 - j]).value(num)
        end
        p_insurance['PublicInsuredPerson_Number'].to_s.zenkaku.split(//)[0, 7].each_with_index do |num, j|
          page.item('PublicInsuredPerson_Number%d_%d' % [i, 7 - j]).value(num)
        end
      end

      data['Disease_Information'][0, FIRST_PAGE_DISEASE_ROWS].each.with_index(1) do |disease_row, j|

        disease = edit_disease_row(disease_row)

        page.item('Disease_Name%02d' % j).value(disease['Name'])
        page.item('Disease_StartDate%02d' % j).value(disease['StartDate'])
        page.item('Disease_EndDate%02d' % j).value(disease['EndDate'])
        page.item('Cure%02d' % j).value(disease['Cure'])
        page.item('Death%02d' % j).value(disease['Death'])
        page.item('Discontinued%02d' % j).value(disease['Discontinued'])
      end
    end
  end

  def edit_middle_page(report, data, start_row)

    report.start_new_page layout: :middle do |page|
      patient = data['Patient']
      page.item("Patient_ID").value(patient['ID'])
      page.item("Name").value(patient['Name'])

      data['Disease_Information'][start_row, MIDDLE_PAGE_DISEASE_ROWS].each.with_index(1) do |disease_row, j|

        disease = edit_disease_row(disease_row)

        page.item('Disease_Name%02d' % j).value(disease['Name'])
        page.item('Disease_StartDate%02d' % j).value(disease['StartDate'])
        page.item('Disease_EndDate%02d' % j).value(disease['EndDate'])
        page.item('Cure%02d' % j).value(disease['Cure'])
        page.item('Death%02d' % j).value(disease['Death'])
        page.item('Discontinued%02d' % j).value(disease['Discontinued'])
      end
    end
  end
end

