# coding : utf-8

require 'monpe'
require 'pp'
require_relative '../lib/orca-string'
require_relative '../lib/orca-wareki'

class Karte_no1_n

  FIRST_PAGE_DISEASE_ROWS = 10
  MIDDLE_PAGE_DISEASE_ROWS = 22
  RED_FILE = 'HCN01.red'
  RED_FILE2 = 'HCN011.red'

  def export(fname, form_data, opt)
    offset_set = opt[:pr_conf][:offset_settings]
    if offset_set
      offset_1 =  offset_set[RED_FILE.to_sym]
      offset_n =  offset_set[RED_FILE2.to_sym]
    else
      offset_1 = nil
      offset_n = nil
    end

    forms = []
    form_data['Forms'].each do |hash|
      @data = hash['data']
      if @data['Order_Class'] == '2'
        f = make_later_page
        f.set_offset(*offset_n) if offset_n
        forms << f
        next
      end

      ary = edit_ary
      f = make_1st_page(ary)
      f.set_offset(*offset_1) if offset_1
      forms << f
      start_row = FIRST_PAGE_DISEASE_ROWS
      while ary.size  > start_row
        f = make_later_page(ary, start_row)
        f.set_offset(*offset_n) if offset_n
        forms << f
        start_row += MIDDLE_PAGE_DISEASE_ROWS
      end
    end

    Monpe.export(forms,fname)
  end

  def edit_ary
    ary = Array.new

    Array(@data['Disease_Information']).delete_if(&:empty?).each do |hash|
      ary.push(hash) unless hash['Name'].to_s.empty?
    end

    return ary
  end

  def make_1st_page(ary)
    red = File.join(File.dirname(File.expand_path(__FILE__)),RED_FILE)
    form = Monpe::Report.new(red)
    embed_data = form.get_embed_data

    set_head(embed_data)

    unless ary.nil?
      FIRST_PAGE_DISEASE_ROWS.times do |i|
        disease = ary[i]
        next unless disease
        idx = i
        set_disease_row(embed_data,disease,idx)
      end
    end

    form.set_embed_data(embed_data)
    return form
  end

  def set_head(embed_data)

    patient = @data['Patient'] || {}
    embed_data['PTNUM']    = patient['ID']
    embed_data['KANANAME'] = patient['KanaName'].to_s[0,15]
    if patient['Name'].to_s.size > 15
      embed_data['NAME2'] = patient['Name'].to_s[0,30]
    else
      embed_data['NAME'] = patient['Name'].to_s[0,15]
    end
    unless patient['BirthDate'].to_s.strip.empty?
      ymd = Orca::Wareki.new(patient['BirthDate'])
      embed_data['BIRTHYMD.BIRTHGG'] = ymd.gengoN
      embed_data['BIRTHYMD.BIRTHYY'] = '%2d' % ymd.year_w
      embed_data['BIRTHYMD.BIRTHMM'] = '%2d' % ymd.month
      embed_data['BIRTHYMD.BIRTHDD'] = '%2d' % ymd.day
      embed_data['NENREI'] = ymd.age
    end
    embed_data['SEX[0]'] = '○' if patient['Sex'] == '1'
    embed_data['SEX[1]'] = '○' if patient['Sex'] == '2'

    home_address = patient['Home_Address'] || {}
    embed_data['TEL'] = home_address['PhoneNumber']
    embed_data['ADRS'] = Array(home_address['Address']).join[0, 50]

    contact_address = patient['Contact_Address'] || {}
    embed_data['RENNAME'] = contact_address['Name'].to_s[0, 29]
    embed_data['HIRUTEL'] = contact_address['PhoneNumber_Day']
    embed_data['YORUTEL'] = contact_address['PhoneNumber_Night']
    embed_data['RENADRS'] = Array(contact_address['Address']).join[0 ,58]

    office_address = patient['Office_Address'] || {}
    embed_data['KINNAME'] = office_address['Name'].to_s[0, 29]
    embed_data['KINTEL'] = office_address['PhoneNumber']
    embed_data['KINADRS'] = Array(office_address['Address']).join[0 ,58]

    embed_data['SINRYOKA'] = @data['Department_Name'].to_s[0, 5]

    doctor = @data['Doctor'] || []
    doctor.each_with_index do |dr, i|
      embed_data["DR[#{i}]"] = dr['Name'].to_s[0, 13]
    end

    insurance = @data['Insurance_Combination_Information'] || {}
    h_insurance = insurance['HealthInsurance_Information'] || {}
    a_insurance = insurance['Accident_Insurance_Information'] || {}

    if ! ['9999' , '' ].include?(h_insurance['Certificate_GetDate'].to_s[0,4].strip)
      ymd = Orca::Wareki.new(h_insurance['Certificate_GetDate'])
      embed_data['SYUTOKYMD.SYUTOKGG'] = ymd.gengoN
      embed_data['SYUTOKYMD.SYUTOKYY'] = '%2d' % ymd.year_w
      embed_data['SYUTOKYMD.SYUTOKMM'] = '%2d' % ymd.month
      embed_data['SYUTOKYMD.SYUTOKDD'] = '%2d' % ymd.day
    end

    symbol = h_insurance['HealthInsuredPerson_Symbol'].to_s
    number = h_insurance['HealthInsuredPerson_Number'].to_s
    symbol_number = "#{symbol}#{number}".empty? ? "" :  "#{symbol}・#{number}"
    case insurance['InsuranceProvider_Class']
    when '975'
      embed_data["HPTNUM"] = insurance['InsuranceProvider_Name'].to_s[0, 8]
      embed_data["KIGONUM"] = symbol_number[0, 26]
    when '971', '973'
      embed_data["HPTNUM"] = symbol_number[0, 8]
      embed_data["KIGONUM"] = a_insurance['Accident_Insurance_Number'].to_s[0, 26].zenkaku
    else
      embed_data["HPTNUM"] = h_insurance['InsuranceProvider_Number'].to_s.zenkaku.rjust(8,"　")
      embed_data["KIGONUM"] = symbol_number[0, 26]
    end

    embed_data['HIHKNNAME'] = h_insurance['HealthInsuredPerson_Name'].to_s[0,13]
    embed_data['ZOKUGARA'] = patient['Relationship'].to_s[0,13]
    embed_data['HONKZK'] = '本人' if h_insurance['RelationToInsuredPerson'] == '1'
    embed_data['HONKZK'] = '家族' if h_insurance['RelationToInsuredPerson'] == '2'

    if ! ['9999' , '' ].include?(h_insurance['Certificate_ExpiredDate'].to_s[0,4].strip)
      ymd = Orca::Wareki.new(h_insurance['Certificate_ExpiredDate'])
      embed_data['YUKOYMD.YUKOGG'] = ymd.gengoN
      embed_data['YUKOYMD.YUKOYY'] = '%2d' % ymd.year_w
      embed_data['YUKOYMD.YUKOMM'] = '%2d' % ymd.month
      embed_data['YUKOYMD.YUKODD'] = '%2d' % ymd.day
    end

    embed_data['JIGYONAME'] = h_insurance['InsuranceProvider_Name'].to_s[0,26]
    h_address = h_insurance['InsuranceProvider_Address'] || []
    embed_data['JIGYOADRS'] = Array(h_insurance['InsuranceProvider_Address']).join[0, 26]
    if insurance['PublicInsurance_Information']
      2.times do |i|
        p_insurance = insurance['PublicInsurance_Information'][i]
        p_num = p_insurance['PublicInsurer_Number'].to_s
        pp_num = p_insurance['PublicInsuredPerson_Number'].to_s
        if p_insurance['PublicInsurance_Class'] == '970'
          unless pp_num.empty?
            embed_data['COMMENT'] = "#{p_insurance['PublicInsurance_Name']}（#{pp_num}）"
          else
            embed_data['COMMENT'] = p_insurance['PublicInsurance_Name']
          end
        else
          if p_num.size <= 8
            embed_data["KOH-TBL[#{i}].FTNSNO"] = p_num.zenkaku
          else
            embed_data["KOH-TBL[#{i}].FTNSNON"] = p_num.to_s[0,10].zenkaku
          end
          if pp_num.size <= 7
            embed_data["KOH-TBL[#{i}].JKYSNO"] = pp_num.zenkaku
          else
            embed_data["KOH-TBL[#{i}].JKYSNON"] = pp_num.to_s[0,10].zenkaku
          end
          if ! ['9999' , '' ].include?(p_insurance['Certificate_ExpiredDate'].to_s[0,4].strip)
            ymd = Orca::Wareki.new(p_insurance['Certificate_ExpiredDate'])
            embed_data["KOH-TBL[#{i}].KOHENDYMD.KOHENDGG"] = ymd.gengoN
            embed_data["KOH-TBL[#{i}].KOHENDYMD.KOHENDYY"] = '%2d' % ymd.year_w
            embed_data["KOH-TBL[#{i}].KOHENDYMD.KOHENDMM"] = '%2d' % ymd.month
            embed_data["KOH-TBL[#{i}].KOHENDYMD.KOHENDDD"] = '%2d' % ymd.day
          end
        end
      end
    end
    
    if ! ['9999' , '' ].include?(@data['Admission_Date'].to_s[0,4].strip)
      ymd = Orca::Wareki.new(@data['Admission_Date'])
      embed_data['NYUINYMD.NYUINGG'] = ymd.gengoN
      embed_data['NYUINYMD.NYUINYY'] = '%2d' % ymd.year_w
      embed_data['NYUINYMD.NYUINMM'] = '%2d' % ymd.month
      embed_data['NYUINYMD.NYUINDD'] = '%2d' % ymd.day
    end

    if ! ['9999' , '' ].include?(@data['Discharge_Date'].to_s[0,4].strip)
      ymd = Orca::Wareki.new(@data['Discharge_Date'])
      embed_data['TAIINYMD.TAIINGG'] = ymd.gengoN
      embed_data['TAIINYMD.TAIINYY'] = '%2d' % ymd.year_w
      embed_data['TAIINYMD.TAIINMM'] = '%2d' % ymd.month
      embed_data['TAIINYMD.TAIINDD'] = '%2d' % ymd.day
    end
  end

  def set_disease_row(embed_data,disease,idx)
    embed_data["BYOMEI-TBL[#{idx}].BYOMEI"] = disease['Name'].to_s[0, 50]
    if ! ['9999' , '' ].include?(disease['StartDate'].to_s[0,4].strip)
      ymd = Orca::Wareki.new(disease['StartDate'])
      embed_data["BYOMEI-TBL[#{idx}].STRYMD.STRYY"] = '%2d' % ymd.year_w
      embed_data["BYOMEI-TBL[#{idx}].STRYMD.STRMM"] = '%2d' % ymd.month
      embed_data["BYOMEI-TBL[#{idx}].STRYMD.STRDD"] = '%2d' % ymd.day
    end
    if ! ['9999' , '' ].include?(disease['EndDate'].to_s[0,4].strip)
      ymd = Orca::Wareki.new(disease['EndDate'])
      embed_data["BYOMEI-TBL[#{idx}].ENDYMD.ENDYY"] = '%2d' % ymd.year_w
      embed_data["BYOMEI-TBL[#{idx}].ENDYMD.ENDMM"] = '%2d' % ymd.month
      embed_data["BYOMEI-TBL[#{idx}].ENDYMD.ENDDD"] = '%2d' % ymd.day
    end
    case disease['OutCome_Class']
    when '1'
      embed_data["BYOMEI-TBL[#{idx}].TENKI[0]"] = '○'
    when '2'
      embed_data["BYOMEI-TBL[#{idx}].TENKI[2]"] = '○'
      embed_data["BYOMEI-TBL[#{idx}].TENKIMSG"] = '死'
    when '3'
      embed_data["BYOMEI-TBL[#{idx}].TENKI[1]"] = '○'
    when '8'
      embed_data["BYOMEI-TBL[#{idx}].TENKI[2]"] = '○'
      embed_data["BYOMEI-TBL[#{idx}].TENKIMSG"] = '移'
    end
  end

  def make_later_page(ary = nil, start_row = nil)
    red = File.join(File.dirname(File.expand_path(__FILE__)),RED_FILE2)
    form = Monpe::Report.new(red)
    embed_data = form.get_embed_data

    patient = @data['Patient']
    embed_data['PTNUM'] = patient['ID']
    if patient['Name'].to_s.size > 15
      embed_data['NAME2'] = patient['Name'].to_s[0,30]
    else
      embed_data['NAME'] = patient['Name'].to_s[0,15]
    end

    unless ary.nil?
      MIDDLE_PAGE_DISEASE_ROWS.times do |j|
        i = start_row + j
        disease = ary[i]
        next unless disease
        idx = j
        embed_data["BYOMEI-TBL[#{idx}].CD"] = i + 1
        set_disease_row(embed_data,disease,idx)
      end
    end

    form.set_embed_data(embed_data)
    return form
  end
end

=begin
pdf_file = "karte_no1_n.pdf"

form_data = <<EOF
{
  "Forms": [
    {
      "data":
      {
      }
    }
  ]
}
EOF

conf = { :api_user => 'ormaster',
         :api_key => 'ormaster',
         :api_server => 'localhost',
         :api_port => 8000 }
opt = Hash.new
opt[:conf] = conf
require 'json'
form = Karte_no1_n.new
form.export(pdf_file, JSON.parse(form_data), opt)
=end
