# coding : utf-8

require 'monpe'
require 'pp'
require_relative '../lib/orca-string'
require_relative '../lib/orca-wareki'

class Karte_no1m

  FIRST_PAGE_DISEASE_ROWS = 8
  MIDDLE_PAGE_DISEASE_ROWS = 22

  # call from print001.rb
  def export(fname,report_data,opt)
    reports = []
    report_data['Forms'].each do |form|
      form_data = form['data']
      if form_data['Order_Class'] == '2'
        reports << make_later_page(form_data)
        next
      end
      form_data['Disease_Information']  ||= []
      form_data['Disease_Information'].delete_if do |item| 
        item['Name'].to_s.empty?
      end
      reports << make_1st_page(form_data)
      start_row = FIRST_PAGE_DISEASE_ROWS
      while form_data['Disease_Information'].size  > start_row
        reports << make_later_page(form_data,start_row)
        start_row += MIDDLE_PAGE_DISEASE_ROWS
      end
    end
    Monpe.export(reports,fname)
  end


  def make_1st_page(data)
    red = File.join(File.dirname(File.expand_path(__FILE__)),'HCM01.red')
    report = Monpe::Report.new(red)
    embed = report.get_embed_data

    patient = data['Patient'] || {}
    hospital = data['Hospital'] || {}
    insurance = data['Insurance_Combination_Information'] || {}
    h_insurance = insurance['HealthInsurance_Information'] || {}
    a_insurance = insurance['Accident_Insurance_Information'] || {}

    embed['PTNUM']    = patient['ID']
    if patient['KanaName'].size > 15
      embed['KANANAME2'] = patient['KanaName']
    else
      embed['KANANAME'] = patient['KanaName']
    end
    if patient['Name']
      embed['NAME2'] = patient['Name']
    else
      embed['NAME'] = patient['Name']
    end
    ymd = Orca::Wareki.new(patient['BirthDate'])
    embed['TANJYOG'] = ymd.gengoN
    embed['TANJYOY'] = '%2d' % ymd.year_w
    embed['TANJYOM'] = '%2d' % ymd.month
    embed['TANJYOD'] = '%2d' % ymd.day
    embed['NENREI'] = ymd.age
    case patient['Sex']
    when '1'
      embed['OTOKO'] = '○'
    when '2'
      embed['ONNA'] = '○'
    end

    home_address = patient['Home_Address'] || {}
    embed['ADRS'] = [home_address['Address']].join
    embed['ADRSTEL'] = home_address['PhoneNumber']
    embed['WORK'] = patient['Job']
    embed['TUZUKI'] = patient['Relationship']
    if data['Memo']
      5.times do |i|
        embed["COMMENT[#{i}]"] = data['Memo'][i] || ""
      end
    end

    case insurance['InsuranceProvider_Class']
    when '975'
      embed['HPTNUM'] = insurance['Insurance_Name']
    when '971', '973'
      hptnum = h_insurance['HealthInsuredPerson_Symbol'] + "・　" + h_insurance['HealthInsuredPerson_Number']
      embed['HPTNUM'] = hptnum
      embed['KIGONUM'] = a_insurance['Accident_Insurance_Number']
    else
      if insurance['InsuranceProvider_Class'] == '039'
        embed['KIGONUM'] = h_insurance['HealthInsuredPerson_Symbol']
      else
        if h_insurance['HealthInsuredPerson_Symbol'] and h_insurance['HealthInsuredPerson_Number']
          kigonum = h_insurance['HealthInsuredPerson_Symbol'] + "・" + h_insurance['HealthInsuredPerson_Number']
          if kigonum.size > 32
            embed['KIGO'] = h_insurance['HealthInsuredPerson_Symbol']
            embed['NUM']  = h_insurance['HealthInsuredPerson_Number']
          else
            embed['KIGONUM'] = kigonum
          end
        end
      end
    end

    if ! ['9999' , '' ].include?(h_insurance['Certificate_ExpiredDate'].to_s[0,4].strip)
      ymd = Orca::Wareki.new(h_insurance['Certificate_ExpiredDate'])
      embed['YUKOG'] = ymd.gengoN
      embed['YUKOY'] = '%2d' % ymd.year_w
      embed['YUKOM'] = '%2d' % ymd.month
      embed['YUKOD'] = '%2d' % ymd.day
    else
      ymd = Orca::Wareki.new
      embed['YUKOG'] = ymd.gengoN
    end
    embed['HIHKNJNAME'] = h_insurance['HealthInsuredPerson_Name']
    if ! ['9999' , '' ].include?(h_insurance['Certificate_GetDate'].to_s[0,4].strip)
      ymd = Orca::Wareki.new(h_insurance['Certificate_GetDate'])
      embed['SIKAKUG'] = ymd.gengoN
      embed['SIKAKUY'] = '%2d' % ymd.year_w
      embed['SIKAKUM'] = '%2d' % ymd.month
      embed['SIKAKUD'] = '%2d' % ymd.day
    else
      ymd = Orca::Wareki.new
      embed['SIKAKUG'] = ymd.gengoN
    end

    office_address = patient['Office_Address'] || {}
    embed['KINADRS'] = [office_address['Address']].join
    embed['KINADRSTEL'] = office_address['PhoneNumber']
    embed['KINNAME'] = office_address['Name']

    embed['HKNADRS'] = h_insurance['InsuranceProvider_Address']
    embed['HKNADRSTEL'] = h_insurance['InsuranceProvider_PhoneNumber']
    embed['HKNNAME'] = h_insurance['InsuranceProvider_Name'] 

    if insurance['PublicInsurance_Information']
      2.times do |i|
        p_insurance = insurance['PublicInsurance_Information'][i]
        p_num = p_insurance['PublicInsurer_Number'].to_s
        idx = i + 1
        if p_num.size <= 8
          embed["FKOH-TBL[#{idx}].FTNSNO"] = p_num
        else
          embed["FKOHN-TBL[#{idx}].FTNSNON"] = p_num
        end
        pp_num = p_insurance['PublicInsuredPerson_Number'].to_s
        if pp_num.size <= 7
          embed["JKOH-TBL[#{idx}].JKYSNO"] = p_num
        elsif pp_num.size <= 10
          embed["JKOHN-TBL[#{idx}].JKYSNON"] = p_num
        else
          embed["JKOH12-TBL[#{idx}].JKYSNO12"] = p_num
        end
      end
    end

    FIRST_PAGE_DISEASE_ROWS.times do |i|
      disease = data['Disease_Information'][i]
      next unless disease
      idx = i
      set_disease_row(embed,disease,idx)
    end

    #pp embed

    report.set_embed_data(embed)
    report
  end

  def set_disease_row(embed,disease,idx)
    embed["BYOMEI-TBL[#{idx}].BYOMEIG.BYOMEI"] = disease['Name']
    ymd = Orca::Wareki.new(disease['StartDate'])
    embed["STRYMD-TBL[#{idx}].STRYMDG.STRYMD[0]"] = '%2d' % ymd.year_w
    embed["STRYMD-TBL[#{idx}].STRYMDG.STRYMD[1]"] = '%2d' % ymd.month
    embed["STRYMD-TBL[#{idx}].STRYMDG.STRYMD[2]"] = '%2d' % ymd.day
    if disease['EndDate']
      ymd = Orca::Wareki.new(disease['EndDate'])
      embed["ENDYMD-TBL[#{idx}].ENDYMDG.ENDYMD[0]"] = '%2d' % ymd.year_w
      embed["ENDYMD-TBL[#{idx}].ENDYMDG.ENDYMD[1]"] = '%2d' % ymd.month
      embed["ENDYMD-TBL[#{idx}].ENDYMDG.ENDYMD[2]"] = '%2d' % ymd.day
    end
    case disease['OutCome_Class']
    when '1'
      embed["TENKI-TBL[#{idx}].TENKIG.TENKI[0]"] = '○'
    when '2'
      embed["TENKI-TBL[#{idx}].TENKIG.TENKI[1]"] = '○'
    when '3'
      embed["TENKI-TBL[#{idx}].TENKIG.TENIKI[2]"] = '○'
    end
  end

  def make_later_page(data, start_row = nil)
    red = File.join(File.dirname(File.expand_path(__FILE__)),'HCM011.red')
    report = Monpe::Report.new(red)
    embed = report.get_embed_data

    patient = data['Patient']
    embed['PTNUM'] = patient['ID']
    if patient['Name'].size > 15
      embed['NAME2'] = patient['Name']
    else
      embed['NAME'] = patient['Name']
    end

    if start_row
      MIDDLE_PAGE_DISEASE_ROWS.times do |j|
        i = start_row + j
        disease = data['Disease_Information'][i]
        next unless disease
        idx = j
        set_disease_row(embed,disease,idx)
      end
    end

    #pp embed

    report.set_embed_data(embed)
    report
  end
end

