# coding : utf-8

require 'monpe'
require_relative '../lib/orca-string'
require_relative '../lib/orca-wareki'

class Integer
  def with_comma
    self.to_s.gsub(/(\d)(?=(\d{3})+(?!\d))/, '\1,')
  end
end

class Meisaisho_n

  def wareki_n(ymd)
    ymd.to_s.strip.empty? ? '' : Orca::Wareki.wareki_n(ymd)
  end

  LINE_ROW_SIZE  = 69
  DESCRIPTION_SIZE  = 38
  RED_FILE  = 'HCN04.red'

  def export(fname, form_data, opt)
    offset_set = opt[:pr_conf][:offset_settings]
    if offset_set
      offset_1 =  offset_set[RED_FILE.to_sym]
    else
      offset_1 = nil
    end

    forms = []

    form_data['Forms'].each do |hash|
      @data = hash['data']
      @srykbn = ''
      ary = edit_ary
      ary.each_slice(LINE_ROW_SIZE).each.with_index(1) do |ary, page_no|
        f = edit_page(ary, page_no)
        f.set_offset(*offset_1) if offset_1
        forms << f
      end
    end

    forms << edit_page if forms.size == 0

    Monpe.export(forms, fname)
  end

  def edit_ary
    ary = Array.new

    Array(@data['Statement']).delete_if(&:empty?).each do |hash|
      des = 'Description'
      if hash[des].nil? || hash[des].size <= DESCRIPTION_SIZE
        ary.push(hash)
      else
        rows = Array.new
        rows.push({des => hash[des][0,DESCRIPTION_SIZE]})
        slice_size = DESCRIPTION_SIZE - 1
        rows.concat(hash[des][DESCRIPTION_SIZE..-1].each_char.each_slice(slice_size).map{|s| {des => '　' + s.join}})
        hash.each do |k, v|
          rows.first[k] = v if ['Medical_Class_Name'].include?(k)
          rows.last[k] = v unless ['Description', 'Medical_Class_Name'].include?(k)
        end
        ary.concat(rows)
      end
      ary.push({des => ''}) if (ary.last['End_Flg'] == '1' and (ary.size % LINE_ROW_SIZE) != 0 )
    end
    ary.last['Last'] = '1'

    return ary
  end

  def edit_page(ary = nil, page_no = nil)
    red = File.join(File.dirname(File.expand_path(__FILE__)), RED_FILE)
    form = Monpe::Report.new(red)
    embed_data = form.get_embed_data

    unless ary.nil?
      set_head(embed_data, page_no)
      Array(ary).each_with_index do |hash, i|
        set_list(embed_data, hash, i)
      end
      if ary.last['Last'].nil?
        embed_data["NAIYO-TBL[#{ary.size}].SRYMEI"] = '　' * 14 + '次ページへ続く'
      else
        row = ary.size < LINE_ROW_SIZE ? ary.size + 1 : LINE_ROW_SIZE
        embed_data["NAIYO-TBL[#{row}].SRYMEI"] = '　' * 14 + '以下余白'
      end
    end

    form.set_layer_visible('L1',false)
    form.set_layer_visible('L2',false)
    form.set_layer_visible('L3',false)

    form.set_embed_data(embed_data)
    return form
  end

  def set_head(embed_data, page_no)
    patient = @data['Patient'] || {}
    hospital = @data['Hospital'] || {}

    embed_data['HAKYMD'] = wareki_n(@data['IssuedDate'])

    unless @data['Perform_Date'].nil?
      embed_data['NYUINSTRYMD'] = wareki_n(@data['Perform_Date'])
    end
    unless @data['Perform_End_Date'].nil?
      embed_data['NYUINENDYMD'] = wareki_n(@data['Perform_End_Date'])
    end

    embed_data['PAGE'] = page_no.to_s
    embed_data['NO'] = @data['Invoice_Number']

    embed_data['PTNUM'] = patient['ID']
    embed_data['KANANAME'] = patient['KanaName']
    if patient['Name'].to_s.size > 15
      embed_data['NAME2'] = patient['Name'].to_s[0,25]
    else
      embed_data['NAME'] = patient['Name'].to_s[0,15]
    end

    if (@data['Ward_Name'] or @data['Room_Number'])
      embed_data['ROOM'] = "#{@data['Ward_Name']}　#{@data['Room_Number'].to_s.zenkaku}号室"
    end

    embed_data['SRYKANAME'] = @data['Department_Name'].to_s[0,10]
    embed_data['HOKEN'] = @data['Insurance_Name'].to_s[0,10]
    index = @data['Insurance_Rate'].to_s.index(/(割|％)/)
    unless index.nil?
      embed_data['FUTAN'] = @data['Insurance_Rate'][0, index]
      embed_data['FUTANLABEL'] = @data['Insurance_Rate'][index..-1]
    end

    embed_data['CHUI'] = @data['Caution'].to_s[0,50]

    address = hospital['Address'] || []
    name = hospital['Name'] || []
    if address[1].to_s.empty?
      embed_data['IRYOADRS'] = address[0].to_s[0,50]
    else
      embed_data['IRYOADRS'] = "#{address[0].to_s[0,25].ljust(25,"　")}#{address[1].to_s[0,25]}"
    end
    if name[1].to_s.empty?
      embed_data['IRYOMEI'] = name[0].to_s[0,50]
    else
      embed_data['IRYOMEI'] = "#{name[0].to_s[0,25].ljust(25,"　")}#{name[1].to_s[0,25]}"
    end
    embed_data['IRYOTEL'] = hospital['PhoneNumber']

    if @data['Yen_Flg'] == '1'
      embed_data['LABEL3'] = '　単　価　'
      embed_data['LABEL2'] = '※金額欄については１０割分の表示をしております。患者一部負担金は保険公費による負担割合分となります。'
    else
      embed_data['LABEL3'] = '　点　数　'
      embed_data['LABEL2'] = '※合計欄は点数での表示をしています。患者一部負担金はこの点数より計算（１点＝１０円）し、保険公費による負担割合分となります。'
    end
    embed_data['LABEL1'] = '　　合　　計　　'
  end

  def set_list(embed_data, hash, i)
    if hash['Group_Head_Flg'] == '1'
      embed_data["NAIYO-TBL[#{i}].SRYKBN"] = '-' * 10
      embed_data["NAIYO-TBL[#{i}].KAISU2"] =  '-' * 38
      embed_data["NAIYO-TBL[#{i}].KINGAKU2"] =  '-' * 16
      size = DESCRIPTION_SIZE - hash['Description'].to_s.size
      embed_data["NAIYO-TBL[#{i}].SRYMEI"] = hash['Description'].to_s + ("-" * (size * 2))
      @srykbn = ''
    else
      yen_kanji = ((@data['Yen_Flg'] != '1' and hash['Yen_Flg'] == '1') ? '円' : '')
      if hash['Medical_Class_Name']
        embed_data["NAIYO-TBL[#{i}].SRYKBN"] = hash['Medical_Class_Name']
        @srykbn = hash['Medical_Class_Name']
      else
        embed_data["NAIYO-TBL[#{i}].SRYKBN"] = @srykbn if i == 0
      end
      embed_data["NAIYO-TBL[#{i}].SRYMEI"] = hash['Description']
      unless hash['Point'].nil?
        embed_data["NAIYO-TBL[#{i}].TENSU"] = "#{hash['Point'].to_i.with_comma}#{yen_kanji}"
      end
      embed_data["NAIYO-TBL[#{i}].KAISU"] = hash['Count'].to_i.with_comma unless hash['Count'].nil?
      embed_data["NAIYO-TBL[#{i}].KINGAKU"] = "#{hash['Total_Point'].to_i.with_comma}#{yen_kanji}" unless hash['Total_Point'].nil? 
    end
  end

end

=begin
pdf_file = "meisaisho_n.pdf"

form_data = <<EOF
{
  "Forms": [
    {
      "data":
      {
      }
    }
  ]
}
EOF

conf = { :api_user => 'ormaster',
         :api_key => 'ormaster',
         :api_server => 'localhost',
         :api_port => 8000 }
opt = Hash.new
opt[:conf] = conf
require 'json'
form = Meisaisho_n.new
form.export(pdf_file, JSON.parse(form_data), opt)
=end
