# coding : utf-8
require 'uri'
require 'net/http'
require 'fileutils'
require 'zip'
require 'push_exchanger'
require 'monpe'
require 'pp'
require_relative '../lib/orca-wareki'
require_relative '../lib/orca-string'


class Okusuri_joho_n

  IMAGE_DIR_LINUX = '/var/tmp/print001/okusuri_joho'
  IMAGE_DIR_WINDOWS =  "#{ENV['USERPROFILE']}/.push-exchanger/tmp/print001/okusuri_joho"

  FRAMES_IN_PAGE = 8
  LINES_IN_FRAME = 8
  RED_FILE  = 'HCN300.red'


  def wareki_n(ymd)
    ymd.to_s.strip.empty? ? '' : Orca::Wareki.wareki_n(ymd)
  end

  def get_image_dir
    if RUBY_PLATFORM.downcase =~ /mswin(?!ce)|mingw|cygwin|bccwin/
      return IMAGE_DIR_WINDOWS
    else
      return IMAGE_DIR_LINUX
    end
  end

  def get_image(srycd, image_id)

    return nil if srycd.to_s.empty?

    begin
      image_dir = get_image_dir
      FileUtils.mkdir_p(image_dir) unless FileTest.exist?(image_dir)
      image_file = Dir.glob("#{image_dir}/#{srycd}*").first

      return image_file unless image_file.nil?
      return nil if image_id.to_s.empty?

=begin
res = PushExchanger::Util::http_post(@opt[:conf],"/api01rv2/imagegetv2",data)
raise "error" unless res.code == "200"
p res.body
p res.content-type
p res["X-HOGEHOGE-HEADER"]
=end

      path = "/api01rv2/imagegetv2"
      xml = <<-EOF
        <data>
          <imagegetv2req type="record">
            <Image_ID type="string">#{image_id}</Image_ID>
          </imagegetv2req>
        </data>
      EOF
      res = PushExchanger::Util::http_post(@opt[:conf],"/api01rv2/imagegetv2",xml)
      unless res.code == "200"
        raise "http error:#{path} #{res.code}"
      end

      Zip::File.open_buffer(res.body) do |zf|
        zf.each do |entry|
          image_file = "#{image_dir}/#{entry.name}"
          entry.extract(image_file){true}
        end
      end

      return image_file
    rescue
      nil
    end
  end

  def get_max(ary)
   ary.max.quo(LINES_IN_FRAME).ceil * LINES_IN_FRAME
  end

  def edit_draft

    draft=Array.new

    Array(@data['Medicine_Information']).each.with_index(1) do |m, i|
    
      break if m['Name'].nil?
      info = Hash.new
      info['No'] = i
      info['Effect'] = Array.new
      info['Style'] = Array.new
      info['Usage'] = Array.new
      info['Caution'] = Array.new
      info['Code'] = m['Code'] || ''

      if m['Dosage'].kind_of?(Hash)
        ['Risingtime', 'Morning', 'Afternoon', 'Evening', 'Bedtime'].each do |d|
          u = "#{d}_Unit_Name"
          info[d] =  m['Dosage'][d]
          info[u] =  m['Dosage'][u]
        end
      end

      info['Effect'].concat(adjust(m['Name'].to_s[0,48], 16))
      info['Effect'].concat([''] * (3 - [3, info['Effect'].size % LINES_IN_FRAME].min))
      info['Effect'].concat(adjust(m['Effect'].to_s ,17))

      info['Image_ID'] = get_image(m['Code'], m['Image_ID'])

      style = Array.new
      unless info['Image_ID'].nil?
        style.concat([''] * 6)
      end

      info['Usage'].concat([''] * 2)
      info['Usage'].concat(adjust(Array(m['Usage']).join("\n"), 10))
      info['Caution'].concat(adjust(m['Caution'].to_s.rstrip, 22))

      max = get_max([info['Effect'].size, info['Usage'].size, info['Caution'].size])
      if info['Image_ID'].nil?
        style.concat(adjust(m['Color'].to_s.rstrip, 10))
        style.concat(adjust(m['Shape'].to_s.rstrip, 10))
        style.concat(adjust(m['Mark'].to_s.rstrip, 10))
      else
        if ['2', '3'].include?(@data['Style_Class'].to_s)
          style.concat(adjust(m['Color'].to_s.rstrip, 10))
          style.concat(adjust(m['Mark'].to_s.rstrip, 10))
        end
      end
      style.slice!(max..-1) if ['0', '2'].include?(@data['Style_Class'].to_s)
      info['Style'].concat(style)
      max = get_max([max, info['Style'].size])

      info['Max'] = max

      draft.push(info)

    end
    
    return draft
    
  end

  def adjust(text, size)
    text.scan(/.{1,#{size}}/).flat_map{|line| line.split("\n")}
  end

  def edit_top_and_bottom(embed_data)
    @page += 1
    embed_data['PAGE'] = @page.to_s
    patient = @data['Patient'] || {}
    hospital = @data['Hospital'] || {}
    doctor = @data['Doctor'] || {}

    embed_data['SYURUI'] = "（#{@data['Medicine_Count'].to_i}種類）"
    embed_data['ID'] = patient['ID']
    embed_data['KANANAME'] = patient['KanaName']
    case patient['Name'].to_s.size
    when 0..15
      embed_data['NAME'] = "#{patient['Name']}　様"
    when 16..20
      embed_data['NAME2'] = "#{'　' * 22}#{patient['Name']}　様"
    else
      embed_data['NAME2'] = "#{patient['Name'].to_s.ljust(43,'　')}様"
    end
    embed_data['SYORIBI'] = wareki_n(@data['IssuedDate'])
    embed_data['BTUNAME'] = "#{@data['Ward_Name'].to_s.strip}#{@data['Room_Number'].to_s}"

    address = hospital['Address'] || []
    name = hospital['Name'] || []
    if address[1].to_s.empty?
      embed_data['JYUSYO'] = address[0].to_s[0,50]
    else
      embed_data['JYUSYO'] = "#{address[0].to_s[0,25].ljust(25,"　")}#{address[1].to_s[0,25]}"
    end
    if name[1].to_s.empty?
      embed_data['HSPNAME'] = name[0].to_s[0,50]
    else
      embed_data['HSPNAME'] = "#{name[0].to_s[0,25].ljust(25,"　")}#{name[1].to_s[0,25]}"
    end
    embed_data['TEL'] = hospital['PhoneNumber']
    embed_data['DRNAME'] = doctor['Name'].to_s[0,13]

  end

  def is_page_full?
    @fno >= FRAMES_IN_PAGE
  end

  def initialize_form(form ,embed_data)

    form_dir = File.dirname(File.expand_path(__FILE__))
    dummy_image = File.join(form_dir, 'transparent.png')

    @fno = 0
    (1..FRAMES_IN_PAGE).each do |i|
      form.set_layer_visible("ST#{i}", false)
      form.set_layer_visible("WK#{i}", false)
      form.set_layer_visible("ED#{i}", false)
      embed_data["TBL[#{i - 1}].GAZO"] = dummy_image
    end
    edit_top_and_bottom(embed_data)
  end

  def finalize_form(form ,embed_data ,offset_1)
    form.set_layer_visible("ED#{@fno}", true)
    form.set_embed_data(embed_data)
    form.set_offset(*offset_1) if offset_1
    return form
  end

  def embed_dosage(embed_data, info, in_key , out_key)
    embed_data["TBL[#{@fno}].KBN-G.KBN[#{out_key}]"] = (info[in_key].to_s.size == 1 ? info[in_key].to_s.zenkaku : info[in_key])
    embed_data["TBL[#{@fno}].TANI-G.TANI[#{out_key}]"] = info["#{in_key}_Unit_Name"]
  end

  def edit_form(draft ,offset_1)

    forms = Array.new

    form_dir = File.dirname(File.expand_path(__FILE__))
    red_file = File.join(form_dir, RED_FILE)
    form = Monpe::Report.new(red_file)

    embed_data = form.get_embed_data
    initialize_form(form ,embed_data)
    draft.each_with_index do |info|
      if is_page_full?
        forms.push(finalize_form(form ,embed_data ,offset_1))
        form = Monpe::Report.new(red_file)
        embed_data = form.get_embed_data
        initialize_form(form ,embed_data)
      end

      embed_dosage(embed_data, info, 'Risingtime', "4")
      embed_dosage(embed_data, info, 'Morning', "0")
      embed_dosage(embed_data, info, 'Afternoon', "1")
      embed_dosage(embed_data, info, 'Evening', "2")
      embed_dosage(embed_data, info, 'Bedtime', "3")

      form.set_layer_visible("ED#{@fno}", true) if @fno > 0
      form.set_layer_visible("ST#{@fno + 1}", true)
      form.set_layer_visible("WK#{@fno + 1}", true)

      embed_data["TBL[#{@fno}].GAZO"] = info['Image_ID'] unless info['Image_ID'].nil?

      lno = 0
      embed_data["TBL[#{@fno}].NUM"] = info['No']
      info['Max'].times do |i|
        if lno >= LINES_IN_FRAME
          lno = 0
          @fno += 1
          if is_page_full?
            forms.push(finalize_form(form ,embed_data ,offset_1))
            form = Monpe::Report.new(red_file)
            embed_data = form.get_embed_data
            initialize_form(form ,embed_data)
          end
          form.set_layer_visible("WK#{@fno + 1}", true)
        end
        if i < 3
          embed_data["TBL[#{@fno}].YKZNAME-G.YKZNAME[#{lno}]"] = info['Effect'][i]
        else
          embed_data["TBL[#{@fno}].KONOKOKA-G.KONOKOKA[#{lno}]"] = info['Effect'][i]
        end
        embed_data["TBL[#{@fno}].KEIJYO-G.KEIJYO[#{lno}]"] = info['Style'][i]
        embed_data["TBL[#{@fno}].NAIYO-G.NAIYO[#{lno}]"] = info['Usage'][i]
        embed_data["TBL[#{@fno}].CYUJKO-G.CYUJKO[#{lno}]"] = info['Caution'][i]
        lno += 1
      end
      form.set_layer_visible("WK#{@fno + 1}", true)
      @fno += 1
    end

    forms.push(finalize_form(form ,embed_data ,offset_1))

    return forms

  end

  def export(pdf_file, form_data, opt)
    @opt = opt
    @page = 0
    @fno = 0

    offset_set = opt[:pr_conf][:offset_settings]
    if offset_set
      offset_1 =  offset_set[RED_FILE.to_sym]
    else
      offset_1 = nil
    end

    forms = Array.new

    form_data['Forms'].each do |hash|
      @data = hash['data']
      draft = edit_draft
      forms.concat(edit_form(draft,offset_1))
    end

    Monpe.export(forms, pdf_file)
  end

end

=begin
pdf_file = "okusuri_joho_n.pdf"
form_data = <<EOF
{
  "Forms": [
    {
      "data": {
      }
    }
  ]
}
EOF

conf = { :api_user => 'ormaster',
         :api_key => 'ormaster',
         :api_server => 'localhost',
         :api_port => 8000 }
opt = Hash.new
opt[:conf] = conf
require 'json'
form = Okusuri_joho_n.new
form.export(pdf_file, JSON.parse(form_data), opt)
=end
