# coding : utf-8

require 'monpe'
require 'pp'
require_relative '../lib/orca-string'
require_relative '../lib/orca-wareki'

class Integer
  def with_comma
    self.to_s.gsub(/(\d)(?=(\d{3})+(?!\d))/, '\1,')
  end
end

class Seikyusho_n

  RED_FILE  = 'HCN03V03A5BN.red'

  def with_comma(val)
    val.to_s.strip.empty? ? '' : val.to_i.with_comma
  end

  def wareki_n(ymd)
    ymd.to_s.strip.empty? ? '' : Orca::Wareki.wareki_n(ymd)
  end

  def edit_top_and_bottom(embed_data)

    patient = @data['Patient'] || {}
    hospital = @data['Hospital'] || {}

    embed_data['NO'] = @data['Invoice_Number']
    embed_data['MIDASHI'] = '（訂正）' if @data['Repirint_Sign_Flg'] == '1'
    embed_data['NYUINSTRYMD'] = wareki_n(@data['Perform_Date'])
    embed_data['NYUINENDYMD'] = wareki_n(@data['Perform_End_Date'])
    embed_data['MIDASI'] = @data['Department_Name']
    if (@data['Ward_Name'] or @data['Room_Number'])
      embed_data['ROOM'] = "#{@data['Ward_Name']}　#{@data['Room_Number'].to_s.zenkaku}号室"
    end
    embed_data['HAKYMD'] = wareki_n(@data['IssuedDate'])
    embed_data['KNJNO'] = patient['ID']
    if patient['Name'].to_s.size > 15
      embed_data['KANANAME2'] = patient['KanaName']
      embed_data['KNJNAMW2'] = patient['Name']
    else
      embed_data['KANANAME'] = patient['KanaName']
      embed_data['KNJNAMW'] = patient['Name']
    end
    embed_data['HOKEN'] = @data['Insurance_Name'].to_s[0, 10]
    index = @data['Insurance_Rate'].to_s.index(/(割|％)/)
    unless index.nil?
      embed_data['FUTAN'] = @data['Insurance_Rate'][0, index]
      embed_data['LBLFTNRATE'] = @data['Insurance_Rate'][index..-1]
    end

    address = hospital['Address'] || []
    name = hospital['Name'] || []
    if address[1].to_s.empty?
      embed_data['IRYOADRS'] = "#{address[0].to_s[0, 50]}"
    else
      embed_data['IRYOADRS'] = "#{address[0].to_s[0, 25].ljust(25, "　")}#{address[1].to_s[0, 25]}"
    end
    if name[1].to_s.empty?
      embed_data['IRYOMEI'] = "#{name[0].to_s[0, 50]}"
    else
      embed_data['IRYOMEI'] = "#{name[0].to_s[0, 25].ljust(25, "　")}#{name[1].to_s[0, 25]}"
    end
    embed_data['IRYOTEL'] = hospital['PhoneNumber']
    embed_data['CHUI'] = @data['Caution']
    Array(@data['Memo']).each_with_index do |memo, i|
       embed_data["MEMO[#{i}]"] = memo
    end
  end

  def edit_form(offset_1)

    form_dir = File.dirname(File.expand_path(__FILE__))
    red_file = File.join(form_dir, RED_FILE)

    form = Monpe::Report.new(red_file)
    embed_data = form.get_embed_data

    edit_top_and_bottom(embed_data)

    Array(@data['Me']).each_with_index do |me, i|
      embed_data["MEISAI[#{i}]"] = me['Name']
      embed_data["HKNRYO[#{i}]"] = with_comma(me['Point']).zenkaku
      embed_data["JIHIBUN[#{i}]"] = with_comma(me['Money']).zenkaku
    end
    embed_data['HKNRYO[16]'] = with_comma(@data['Total_Point']).zenkaku

    embed_data['SKJRYO'] = with_comma(@data['Meal_Cost']).zenkaku
    embed_data['LIFERYO'] = with_comma(@data['Living_Cost']).zenkaku

    embed_data['HOKENBUN'] = with_comma(@data['Ai_Money']).zenkaku
    unless @data['Label_Lsi_Total_Money'].to_s.empty?
      embed_data['RSIMIDASI'] = @data['Label_Lsi_Total_Money']
      embed_data['RSIBUN'] = with_comma(@data['Lsi_Total_Money_In_Ai_Money']).zenkaku
      embed_data['RSIEN'] = '円'
    end
    embed_data['HOKENGAI'] = with_comma(@data['Total_Me_Money']).zenkaku
    embed_data['HOKENZEI'] = with_comma(@data['Tax_In_Total_Me_Money']).zenkaku

    Array(@data['Oe_Etc']).each_with_index do |oe, i|
      embed_data["JIHIMSG[#{i}]"] = oe['Name'].to_s[0,9]
      embed_data["JIHI[#{i}]"] = with_comma(oe['Money']).zenkaku
    end
    embed_data['JIHI[10]'] = with_comma(@data['Total_Oe_Etc_Money']).zenkaku
    embed_data['JIHI[11]'] = with_comma(@data['Tax_In_Total_Oe_Etc_Money']).zenkaku

    embed_data['KOUHI'] = with_comma(@data['Pi_Smoney']).zenkaku
    if (@data['Meal_Smoney'] or @data['Living_Smoney'])
      embed_data['SYOKUJI'] = with_comma("#{@data['Meal_Smoney'].to_i + @data['Living_Smoney'].to_i}").zenkaku
    end

    embed_data['RMSAGAKU'] = with_comma(@data['Room_Charge']).zenkaku
    embed_data['RMZEI'] = with_comma(@data['Tax_In_Room_Charge']).zenkaku

    if (@data['Ad_Money1'] or @data['Ad_Money2'])
      embed_data['CHOSEI'] = with_comma("#{@data['Ad_Money1'].to_i + @data['Ad_Money2'].to_i}").zenkaku
    end

    embed_data['KONSEIKYU'] = with_comma(@data['Ac_Money']).zenkaku
    embed_data['KONZEI'] = with_comma(@data['Tax_In_Ac_Money']).zenkaku
    embed_data['KONNYUKIN'] = with_comma(@data['Ic_Money']).zenkaku
    if (@data['Unpaid_Money_Admission'] or @data['Unpaid_Money_Outpatient'])
      embed_data['LBL_G_MISYU'] = '外 来 未 収 額'
      embed_data['LBL_N_MISYU'] = '入 院 未 収 額'
      embed_data['LBL_G_MISYU_EN'] = '円'
      embed_data['LBL_N_MISYU_EN'] = '円'
    end
    embed_data['NYU_MISYUKIN'] = with_comma(@data['Unpaid_Money_Admission']).zenkaku
    embed_data['GAI_MISYUKIN'] = with_comma(@data['Unpaid_Money_Outpatient']).zenkaku

    embed_data['GAISAN-SEIKYU'] = with_comma(@data['Ac_Money']).zenkaku
    embed_data['GAISAN-ZEI'] = with_comma(@data['Tax_In_Ac_Money']).zenkaku
    embed_data['GAISAN-NMISYU'] = with_comma(@data['Unpaid_Money_Admission']).zenkaku
    embed_data['GAISAN-GMISYU'] = with_comma(@data['Unpaid_Money_Outpatient']).zenkaku

    if @data['Simulation_Flg'] == '1'
      if (@data['Unpaid_Money_Admission'] or @data['Unpaid_Money_Outpatient'])
        form.set_layer_visible('L1',false)
        form.set_layer_visible('L2',false)
        form.set_layer_visible('SKY',false)
      else
        form.set_layer_visible('L1',false)
        form.set_layer_visible('L2',false)
        form.set_layer_visible('G2',false)
        form.set_layer_visible('SKY',false)
      end
    else
      if (@data['Unpaid_Money_Admission'] or @data['Unpaid_Money_Outpatient'])
        form.set_layer_visible('L1',false)
        form.set_layer_visible('G1',false)
        form.set_layer_visible('G2',false)
      else
        form.set_layer_visible('L2',false)
        form.set_layer_visible('G1',false)
        form.set_layer_visible('G2',false)
      end
    end

    form.set_embed_data(embed_data) 
    form.set_offset(*offset_1) if offset_1

    return form

  end

  def export(pdf_file, form_data, opt)

    offset_set = opt[:pr_conf][:offset_settings]
    if offset_set
      offset_1 =  offset_set[RED_FILE.to_sym]
    else
      offset_1 = nil
    end

    forms = Array.new

    form_data['Forms'].each do |hash|
      @data = hash['data']
      forms.push(edit_form(offset_1))
    end

    Monpe.export(forms, pdf_file)
  end

end

=begin
pdf_file = "seikyusho_n.pdf"
form_data = <<EOF
{
  "Forms":
  [
    {
      "data":
      {
      }
    }
  ]
}
EOF
conf = { :api_server => 'localhost',
         :api_port => 8000 }
opt = Hash.new
opt[:conf] = conf
require 'json'
form = Seikyusho_n.new
form.export(pdf_file, JSON.parse(form_data), opt)
=end
