# coding : utf-8

require 'monpe'
require_relative '../lib/orca-string'
require_relative '../lib/orca-wareki'

class Integer
  def with_comma
    self.to_s.gsub(/(\d)(?=(\d{3})+(?!\d))/, '\1,')
  end
end

class Shiharai_shomeisho

  RED_FILE_DAILY_I = 'HCM31NV02.red'
  RED_FILE_DAILY_O = 'HCM31GV02.red'
  RED_FILE_MONTHLY_I = 'HCM32NV02.red'
  RED_FILE_MONTHLY_O = 'HCM32GV02.red'

  MAX_ROWS_DAILY = 30
  MAX_ROWS_MONTHLY = 12

  def with_comma(val)
    val.to_s.strip.empty? ? '' : val.to_i.with_comma
  end

  def wareki_n(ymd)
    ymd.to_s.strip.empty? ? '' : Orca::Wareki.wareki_n(ymd)
  end

  def select_red_file
    if @data['Form_Class'] == '1'
      if @data['InOut'] == '1'
        RED_FILE_DAILY_I
      else
        RED_FILE_DAILY_O
      end
    else
      if @data['InOut'] == '1'
        RED_FILE_MONTHLY_I
      else
        RED_FILE_MONTHLY_O
      end
    end
  end

  def edit_top_and_bottom(embed_data, page)

    patient = @data['Patient'] || {}
    hospital = @data['Hospital'] || {}

    embed_data['STRSRYYMD'] = wareki_n(@data['Start_Date'])
    embed_data['ENDSRYYMD'] = wareki_n(@data['End_Date'])
    embed_data['HAKKOYMD'] = wareki_n(@data['IssuedDate'])
    embed_data['PTNUM'] = patient['ID']
    embed_data['PTNAME'] = patient['Name']
    if @data['Form_Class'] == '2'
      if  @data['Past_Perform_Deal_Class'] == '1'
        embed_data['PAGE'] = "#{page.to_s}頁"
      end
    else
      embed_data['PAGE'] = page.to_s
    end
    address = hospital['Address'] || []
    name = hospital['Name'] || []
    if address[0].to_s.size > 40
      embed_data['HOSPADRS2'] = address[0].to_s[0, 50]
    else
      embed_data['HOSPADRS[0]'] = address[0].to_s[0 ,38]
      embed_data['HOSPADRS[1]'] = address[1].to_s[0 ,38]
    end
    embed_data['HOSPNAME[0]'] = name[0].to_s[0, 40]
    embed_data['HOSPNAME[1]'] = name[1].to_s[0, 40]
    embed_data['HOSPTEL'] = hospital['PhoneNumber']
    if @data['Order_Class'] == '2'
      embed_data['LABELYMD'] = '支払日'
    else
      embed_data['LABELYMD'] = '診療日'
      embed_data['LBLJNISSU1'] = '実日数'
      embed_data['LBLJNISSU2'] = '日'
      embed_data['JNISSU'] = @data['Number_Days']
    end
  end

  def edit_total(embed_data)
    
    pi = @data['Payment_Information'] || {}
    unless pi['Total_Point'].to_i.zero?
      embed_data['GHKNTEN'] = with_comma(pi['Total_Point'])
    end
    unless pi['Total_Ai_Smoney'].to_i.zero?
      embed_data['GFTNMONEY'] = with_comma(pi['Total_Ai_Smoney'])
    end
    unless pi['Total_Meal_Smoney'].to_i.zero?
      embed_data['GSKJMONEY'] = with_comma(pi['Total_Meal_Smoney'])
    end
    unless pi['Total_Oe_Money'].to_i.zero?
      embed_data['GJIHIMONEY'] = with_comma(pi['Total_Oe_Money'])
    end
    unless pi['Total_Ic_Money'].to_i.zero?
      embed_data['GSKYMONEY'] = with_comma(pi['Total_Ic_Money'])
    end
  end

  def last_page?(page)
    page == @last_page
  end

  def edit_form(sheet, page, opt)

    red = File.join(File.dirname(File.expand_path(__FILE__)), select_red_file)
    form = Monpe::Report.new(red)
    embed_data = form.get_embed_data

    offset_set = opt[:pr_conf][:offset_settings]
    if offset_set
      if @data['Form_Class'] == '1'
        if @data['InOut'] == '1'
          offset_1 =  offset_set[RED_FILE_DAILY_I.to_sym]
        else
          offset_1 =  offset_set[RED_FILE_DAILY_O.to_sym]
        end
      else
        if @data['InOut'] == '1'
          offset_1 =  offset_set[RED_FILE_MONTHLY_I.to_sym]
        else
          offset_1 =  offset_set[RED_FILE_MONTHLY_O.to_sym]
        end
      end
    else
      offset_1 = nil
    end

    edit_top_and_bottom(embed_data, page)
    edit_total(embed_data) if last_page?(page)
    sheet.each_with_index do |row, i|

      if @data['Form_Class'] == '1'
        if @data['Order_Class'] == '1' and @data['InOut'] == '1'
          embed_data["SKYSTYMD[#{i}]"] = wareki_n(row['Perform_Date'])
          embed_data["SKYEDYMD[#{i}]"] = "～#{wareki_n(row['Perform_End_Date'])}"
        else
          if  @data['Past_Perform_Deal_Class'] == '1'
            embed_data["SRYYY2[#{i}]"] = wareki_n(row['Perform_Date']).to_s[0,5]
          end
          embed_data["SRYYMD2[#{i}]"] = wareki_n(row['Perform_Date']).to_s[5..-1]
        end
      else
        ymd = "#{row['Perform_Month']}-01"
        if  @data['Past_Perform_Deal_Class'] == '1'
          embed_data["SRYYY2[#{i}]"] = wareki_n(ymd).to_s[0,5]
        end
        embed_data["SRYYYMM2[#{i}]"] = wareki_n(ymd).to_s[5,3]
      end
      unless row['Point'].to_i.zero?
        embed_data["HKNTEN[#{i}]"] = with_comma(row['Point'])
        embed_data["TEN[#{i}]"] = '点'
      end
      unless row['Ai_Smoney'].to_i.zero?
        embed_data["FTNMONEY[#{i}]"] = with_comma(row['Ai_Smoney'])
        embed_data["HYEN[#{i}]"] = '円'
      end
      unless row['Meal_Smoney'].to_i.zero?
        embed_data["SKJMONEY[#{i}]"] = with_comma(row['Meal_Smoney'])
        embed_data["SYEN[#{i}]"] = '円'
      end
      unless row['Oe_Money'].to_i.zero?
        embed_data["JIHIMONEY[#{i}]"] = with_comma(row['Oe_Money'])
        embed_data["JYEN[#{i}]"] = '円'
      end
      unless row['Ic_Money'].to_i.zero?
        embed_data["SKYMONEY[#{i}]"] = with_comma(row['Ic_Money'])
        embed_data["YEN[#{i}]"] = '円'
      end
      unless row['Past_Ic_Money'].to_i.zero?
        embed_data["BFNYUKIN[#{i}]"] = with_comma(row['Past_Ic_Money'])
        embed_data["BFYEN[#{i}]"] = '円'
      end
    end
    form.set_embed_data(embed_data) 
    form.set_offset(*offset_1) if offset_1
    return form
    
  end

  def export(fname, form_data, opt)

    forms = []

    form_data['Forms'].each do |hash|
      @data = hash['data']
      pi = @data['Payment_Information'] || {}
      ary = Array(pi['Payment_List']) || []
      ary.delete_if {|item| item.empty? }
      max_rows = @data['Form_Class'] == '1' ? MAX_ROWS_DAILY : MAX_ROWS_MONTHLY
      slices = ary.each_slice(max_rows).to_a
      @last_page = slices.size
      slices.each.with_index(1) do |sheet, page|
        forms.push(edit_form(sheet, page, opt))
      end
    end
    Monpe.export(forms, fname)
  end
end
