# coding : utf-8

require 'monpe'
require_relative '../lib/orca-string'
require_relative '../lib/orca-wareki'

class Shijisen_n

  PS_SIZE   = 55
  AMOUNT_POS = 37
  NOTE_COL_SIZE = 56
  NOTE_ROW_SIZE = 34
  MEMO_COL_SIZE = 55
  MEMO_ROW_SIZE  = 3
  RED_FILE  = 'HC503.red'


  def to_num(v)
    ((float = Float(v)) && (float % 1.0 == 0) ? float.to_i : float) rescue v
  end

  def wareki_n(ymd)
    ymd.to_s.strip.empty? ? '' : Orca::Wareki.wareki_n(ymd)
  end

  def edit_patient_information(embed_data)

    patient = @data['Patient'] || {}
    doctor = @data['Doctor'] || {}

    embed_data['TITLE'] = @title
    embed_data['PTNUM'] = patient['ID']
    embed_data['KANANAME'] = patient['KanaName'].to_s[0,20]
    embed_data['NAME'] = patient['Name'].to_s[0,20]

    case patient['Sex']
    when '1'
      embed_data['SEX'] = '男'
    when '2'
      embed_data['SEX'] = '女'
    end

    embed_data['BIRTH'] = wareki_n(patient['BirthDate'])
    ymd = Orca::Wareki.new(patient['BirthDate'])

    embed_data['BYOTO'] = @data['Ward_Name'].to_s[0,10]
    embed_data['ROOMNO'] = @data['Room_Number'].to_s[0,6]
    embed_data['SRYYMD'] = wareki_n(@data['Perform_Date'])
    embed_data['SRYKA'] = @data['Department_Name'].to_s[0,10]
    embed_data['DRNAME'] = doctor['Name'].to_s[0,15]
    embed_data['OPID'] = @data['User_ID'].to_s[0,16]

    if @data['Order_Class'] == "1"
      age = ymd.age(@data['IssuedDate'])
      unless age.nil?
        if age > 0
          embed_data['AGE'] = ("%3d才" % age).zenkaku
        else
          unless @data['IssuedDate'].nil? or patient['BirthDate'].nil?
            m = (DateTime.parse(@data['IssuedDate']).strftime('%m%d').to_i
              - DateTime.parse(patient['BirthDate']).strftime('%m%d').to_i) / 100
            embed_data['AGE'] = ("%2dヶ月" % (m < 0 ? 12 + m : m)).zenkaku
          end
        end
      end
      embed_data['SYSYMD'] = @data['IssuedDate'].to_s.delete('-')
      embed_data['TIME'] = @data['IssuedTime'].to_s[0,5]
    end

  end

  def edit_form(note, offset_1)
  
    forms = Array.new
    form_dir = File.dirname(File.expand_path(__FILE__))
    red_file = File.join(form_dir, RED_FILE)

    note['rp'].each_with_index do |rp, i|
      form = Monpe::Report.new(red_file)
      embed_data = form.get_embed_data

      rp ||= Array.new

      rp.each_with_index do |value, j|
        embed_data["NAIYO[#{j}]"] = value
      end

      edit_patient_information(embed_data)

      if @data['Order_Class'] == "1"
        if i < note['rp'].size - 1
          if Array(@data['Rp']).compact.size > 0
            pos = (rp.empty? ? 1 : NOTE_ROW_SIZE)
            embed_data["NAIYO[#{pos}]"] = "#{'　' * 25}次ページへ続く"
          end
        else
          if i > 0 or Array(@data['Rp']).compact.size > 0
            pos = [NOTE_ROW_SIZE, rp.size].min
            embed_data["NAIYO[#{pos}]"] = "#{'　' * 6}以下余白"
          end
        end
      end

      Array(@data['Memo']).each_with_index do |memo, j|
        break if j >= MEMO_ROW_SIZE
        embed_data["BIKOH[#{j}]"] = memo
      end

      form.set_embed_data(embed_data) 
      form.set_offset(*offset_1) if offset_1

      forms.push(form)
    end

    return forms
  end

  def edit_note(draft)

    note = Hash.new

    rp = Array.new

    draft['zai'].each_with_index do |zai, i|
      case i
      when 0
        # do nothing
      when 1
        rp.push('')
      else
        lines = rp.size.modulo(NOTE_ROW_SIZE)
        unless lines == NOTE_ROW_SIZE - 1
          rp.push('--' * NOTE_COL_SIZE) 
        end
        lines += 1
        unless NOTE_ROW_SIZE >= lines + zai['rp'].size + 1
          rp.concat(Array.new(NOTE_ROW_SIZE - lines))
        end
      end
      rp.concat(zai['rp'])
    end

    lines = rp.size.modulo(NOTE_ROW_SIZE)
    rp.push('--' * NOTE_COL_SIZE)  unless lines == 0

    note['rp'] = rp.each_slice(NOTE_ROW_SIZE).to_a
    note['rp'].push(nil) if note['rp'].size == 0

    return note

  end

  def edit_lines(name, amount)
    lines = name.scan(/.{1,#{PS_SIZE}}/)
    unless amount.empty?
      if lines.size == 0 or lines.last.size >= AMOUNT_POS
        lines.push('　' * AMOUNT_POS  + amount)
      else
        lines[-1] = lines.last + '　' * (AMOUNT_POS - lines.last.size) + amount
      end
    end
    return lines
  end

  def edit_draft_zai

    zai = Array.new

    return zai if @data['Rp'].nil?

    @data['Rp'].each do |rp|
      next if rp['Medication'].nil?
      med = Hash.new
      med['rp'] = Array.new
      rp['Medication'].each do |m|
        unless m['Name'].nil?
          amount = to_num(m['Amount']).to_s
          count = amount.index('.') || amount.size
          count = 5 if count > 5
          amount = (' ' * (5 - count) + amount).zenkaku + m['Unit_Name'].to_s
          med['rp'].concat(edit_lines(m['Name'], amount))
        end
      end

      if rp['Count'].to_i > 1
        count = ("%2d回" % rp['Count'].to_i).zenkaku
        med['rp'][-1] = (med['rp'].last + "　" * (PS_SIZE - ( med['rp'].last.size + count.size)) + count)
      end

      med['rp'].map!{|value| "　" + value}
      zai.push(med)
    end

    zai.unshift({'rp' =>["実施日：#{wareki_n(@data['Medical_Date'])}"]}) if zai.count > 0

    return zai

  end

  def edit_draft
    draft = Hash.new
    draft['zai'] = edit_draft_zai
    return draft
  end

  def export(pdf_file, form_data, opt)

    offset_set = opt[:pr_conf][:offset_settings]
    if offset_set
      offset_1 =  offset_set[RED_FILE.to_sym]
    else
      offset_1 = nil
    end

    forms = Array.new
    @opt = opt

    form_data['Forms'].each do |hash|
      @data = hash['data']
      Array(@data['Number_Copies']).each do |nc|
        nc['Copies'].to_i.times do
          @title = nc['Title']
          draft = edit_draft
          note = edit_note(draft)
          forms.concat(edit_form(note, offset_1))
        end
      end
    end

    Monpe.export(forms, pdf_file)
  end
end


=begin
pdf_file = "shijisen_n"
form_data = <<EOF
{
  "Forms": [
    {
      "data": {
      }
    }
  ]
}
EOF
conf = { :api_user => 'ormaster',
         :api_key => 'ormaster',
         :api_server => 'localhost',
         :api_port => 8000 }
opt = Hash.new
opt[:conf] = conf
require 'json'
form = Shijisen_n.new
form.export(pdf_file, JSON.parse(form_data), opt)
=end

