# coding : utf-8

require 'tmpdir'
require 'uri'
require 'net/http'
require 'fileutils'
require 'zip'
require 'push_exchanger'
require 'monpe'
require 'pp'
require_relative '../lib/orca-string'
require_relative '../lib/orca-wareki'

class Shohosen

  PS_SIZE   = 38
  AMOUNT_POS = 21
  NOTE_COL_SIZE = 41
  NOTE_ROW_SIZE  = 18
  MEMO_COL_SIZE = 43
  MEMO_ROW_SIZE  = 4
  MEMO2_ROW_SIZE  = 2
  QR_PER_PAGE = 3
  RED_FILE  = 'HC02QV04.red'
  RED_FILE2 = 'HC02ZA5.red'


  def to_num(v)
    ((float = Float(v)) && (float % 1.0 == 0) ? float.to_i : float) rescue v
  end

  def wareki_n(ymd)
    ymd.to_s.strip.empty? ? '' : Orca::Wareki.wareki_n(ymd)
  end

  def edit_patient_information(embed_data)

    patient = @data['Patient'] || {}
    hospital = @data['Hospital'] || {}
    doctor = @data['Doctor'] || {}
    insurance = @data['Insurance_Combination_Information'] || {}
    h_insurance = insurance['HealthInsurance_Information'] || {}
    a_insurance = insurance['Accident_Insurance_Information'] || {}

    if @data['Split_Count'].to_i.nonzero?
      embed_data['BUNKATULBL.BUNKATULBL-S1'] = '分割指示に係る処方箋'
      embed_data['BUNKATULBL.BUNKATULBL-S2'] = '　＿分割の＿回目'
      embed_data['BUNKATU'] = "#{@data['Split_Count'].to_i}"
      embed_data['KAISUU'] = "#{@data['Split_Number'].to_i}"
    end

    embed_data['PTNUM'] = patient['ID']
    
    if patient['KanaName'].to_s.size > 15
      embed_data['KANANAME2'] = patient['KanaName']
    else
      embed_data['KANANAME'] = patient['KanaName']
    end

    if patient['Name'].to_s.size > 13
      embed_data['NAME2'] = patient['Name']
    else
      embed_data['NAME'] = patient['Name']
    end

    case patient['Sex']
    when '1'
      embed_data['SEX'] = '男'
    when '2'
      embed_data['SEX'] = '女'
    end

    embed_data['BIRTH'] = wareki_n(patient['BirthDate'])

    address = hospital['Address'] || []
    name = hospital['Name'] || []
    if address[1].to_s.empty?
      embed_data['HOSPADRS'] = "#{address[0].to_s[0, 40]}"
    else
      embed_data['HOSPADRS'] = "#{address[0].to_s[0, 20].ljust(20, "　")}#{address[1].to_s[0, 20]}"
    end
    if name[1].to_s.empty?
      embed_data['HOSPNAME'] = "#{name[0].to_s[0, 40]}"
    else
      embed_data['HOSPNAME'] = "#{name[0].to_s[0, 20].ljust(20, "　")}#{name[1].to_s[0, 20]}"
    end

    embed_data['HOSPTEL'] = hospital['PhoneNumber']
    embed_data['DRNAME'] = doctor['Name'].to_s[0,13]

    hospital['Prefectures_Number'].to_s.split(//)[0, 2].each_with_index do |num, i|
      embed_data["PREFNUM.PREFNUM-S[#{i}]"] = num.zenkaku
    end

    embed_data['TENHYOKBN'] = '１'

    hospital['Code'].to_s.split(//)[0, 7].each_with_index do |num, i|
      embed_data["HOSPCD.HOSPCD-S[#{i}]"] = num.zenkaku
    end

    symbol = h_insurance['HealthInsuredPerson_Symbol'].to_s
    number = h_insurance['HealthInsuredPerson_Number'].to_s
    symbol_number = "#{symbol}・#{number}"
    case insurance['InsuranceProvider_Class']
    when '975'
      embed_data["HKNJA"] = insurance['InsuranceProvider_Name'].to_s[0, 8]
    when '971', '973'
      embed_data["HKNJA"] = symbol[0, 8]
      symbol_number = a_insurance['Accident_Insurance_Number'].to_s.zenkaku
    else
      embed_data["HKNJA"] = h_insurance['InsuranceProvider_Number'].to_s.zenkaku.rjust(8,"　")
    end

    unless "#{symbol}#{number}".empty?
      if insurance['InsuranceProvider_Class'] == '039'
        embed_data["KIGONUM"] =  number
      else
        if symbol_number.size > 16
          embed_data["KIGO"] = symbol
          embed_data["NUM"] = number
        else
          embed_data["KIGONUM"] = symbol_number
        end
      end
    end

    case h_insurance['RelationToInsuredPerson']
    when '1'
      embed_data['HONKZKKBN'] = '被保険者'
    when '2'
      embed_data['HONKZKKBN'] = '被扶養者'
    end

    embed_data['KOFYMD'] = wareki_n(@data['IssuedDate'])

    Array(insurance['PublicInsurance_Information'])[0,2].each_with_index do |p_insurance , i|
      if i == 0
        key_f  = 'FTNJA'
        key_jS = 'JKYSNUM'
        key_jM = 'JKYSNUMN'
        key_jL = 'JKYSNUM12'
        key_jLL = 'JKYSNUM15'
      else
        key_f  = 'FTNJA2'
        key_jS = 'JKYSNUM2'
        key_jM = 'JKYSNUM2N'
        key_jL = 'JKYSNUM212'
        key_jLL = 'JKYSNUM215'
      end

      key_j = nil
      embed_data[key_f] = p_insurance['PublicInsurer_Number'].to_s.zenkaku
      case p_insurance['PublicInsuredPerson_Number'].to_s.size
      when 0..7
          key_j = key_jS
      when 8..10
          key_j = key_jM
      when 11..12
          key_j = key_jL
      else
          key_j = key_jLL
      end

      unless key_j.nil?
        embed_data[key_j] = p_insurance['PublicInsuredPerson_Number'].to_s.zenkaku
      end
      
    end

    case @data['Check_Leftover_Class']
    when '1'
      embed_data['CHK1'] = '×'
    when '2'
      embed_data['CHK2'] = '×'
    when '3'
      embed_data['CHK1'] = '×'
      embed_data['CHK2'] = '×'
    end

    embed_data['HKNINAME'] = doctor['Name'].to_s[0,15] if @data['IncludingUnchangeable_Flg'] == '1'
    embed_data['SRYKAMEI'] = @data['Department_Name']

    case insurance['HealthInsuredPerson_Rate_Class']
    when '1'
      embed_data['FTNRATE'] = to_num(insurance['HealthInsuredPerson_Rate']).to_s
      embed_data['FTNWARI'] = '％'
    when '2'
      embed_data['FTNRATE'] = to_num(insurance['HealthInsuredPerson_Rate'].to_i / 10.0).to_s
      embed_data['FTNWARI'] = '割'
    end
  end

  def edit_form(note, offset_1, offset_2)
  
    forms = Array.new

    form_dir = File.dirname(File.expand_path(__FILE__))
    red_file = File.join(form_dir, RED_FILE)
    dummy_image = File.join(form_dir, 'transparent.png')

    note['page'].each_with_index do |page, i|

      form = Monpe::Report.new(red_file)
      embed_data = form.get_embed_data

      page = Hash.new if page.nil?
      rp = page['rp'] || []

      mark = page['mark'] || []

      edit_patient_information(embed_data)
      rp.each_with_index do |value, j|
        embed_data["NAIYO[#{j}]"] = value
        embed_data["HENKOU[#{j}]"] = '×' if mark[j] == '1'
      end

      if i < note['page'].size - 1
        if [@data['Rp']].compact.size > 0
          pos = (rp.empty? ? 1 : NOTE_ROW_SIZE)
          embed_data["NAIYO[#{pos}]"] = "#{'　' * 24}次ページへ続く"
        end
      else
        if i > 0 or [@data['Rp']].compact.size > 0
          pos = [NOTE_ROW_SIZE, rp.size + 1].min
          embed_data["NAIYO[#{pos}]"] = "#{'　' * 6}以下余白"
        end
      end

      note['memo2'].each_with_index do |memo2, j|
        break if j >= MEMO2_ROW_SIZE
        embed_data["BIKOH2[#{j}]"] = memo2
      end

      if i == 0 or note['draft_memo_size'] <= MEMO_ROW_SIZE
        note['memo'].each_with_index do |memo, j|
          break if j >= MEMO_ROW_SIZE
          embed_data["BIKOH[#{j}]"] = memo
        end
      end

      if @data['EditPageNumber_Flg'] == '1'
        embed_data['PAGE'] = "#{i + 1}/#{note['page'].size}"
      end

      embed_data['QR[0]'] = (note['qr'][i * QR_PER_PAGE + 0] || dummy_image)
      embed_data['QR[1]'] = (note['qr'][i * QR_PER_PAGE + 1] || dummy_image)
      embed_data['QR[2]'] = (note['qr'][i * QR_PER_PAGE + 2] || dummy_image)

      form.set_embed_data(embed_data) 
      form.set_offset(*offset_1) if offset_1

      forms.push(form)
    end

    if @data['Split_Count'].to_i.nonzero? and @data['Split_Count'].to_i == @data['Split_Number'].to_i
      red_file = File.join(form_dir, RED_FILE2)
      form = Monpe::Report.new(red_file)
      embed_data = form.get_embed_data
      hospital = @data['Hospital'] || {}
      embed_data['HOSPTEL'] = hospital['PhoneNumber']
      embed_data['HOSPFAX'] = hospital['FaxNumber']

      form.set_embed_data(embed_data) 
      form.set_offset(*offset_2) if offset_2

      forms.push(form)
    end

    return forms
  end

  def get_qr

    qr = Array.new

    return qr if @data['Qr_ID'].nil?
    begin
      path = "/api01rv2/imagegetv2"
      xml = <<-EOF
        <data>
          <imagegetv2req type="record">
            <Image_ID type="string">#{@data['Qr_ID']}</Image_ID>
          </imagegetv2req>
        </data>
      EOF
      res = PushExchanger::Util::http_post(@opt[:conf],"/api01rv2/imagegetv2",xml)
      unless res.code == "200"
        raise "http error:#{path} #{res.code}"
      end

      Zip::File.open_buffer(res.body) do |zf|
        zf.sort.each do |entry|
          extracted_file = "#{@temp_dir}/#{entry.name}"
          entry.extract(extracted_file){true}
          qr.push(extracted_file) if extracted_file =~ /png$/
        end
      end
    rescue => e
      p e
    ensure
      return qr
    end
  end

  def edit_note(draft)

    note = Hash.new
    note['page']  = Array.new
    note['memo2'] = Array.new(draft['memo2'])
    note['memo'] =  Array.new(draft['memo'].select.with_index{|v,i| i < MEMO_ROW_SIZE})
    note['draft_memo_size'] =  Array.new(draft['memo']).size
    note['qr'] = get_qr

    rp = Array.new
    mark = Array.new

    draft['zai'].each do |zai|
      if rp.size == 0 or rp.last.size + zai['rp'].size + 1 > NOTE_ROW_SIZE
        mark.concat(zai['mark'].each_slice(NOTE_ROW_SIZE).to_a)
        rp.concat(zai['rp'].each_slice(NOTE_ROW_SIZE).to_a)
      else
        rp[-1].push('')
        rp[-1].concat(zai['rp'])
        mark[-1].push('')
        mark[-1].concat(zai['mark'])
      end
    end

    if draft['memo'].size > MEMO_ROW_SIZE
      draft['memo'].insert(MEMO_ROW_SIZE, '――　「備考」欄　続き　――')
      rp.concat(draft['memo'][MEMO_ROW_SIZE..-1].each_slice(NOTE_ROW_SIZE).to_a)
    end

    rp.each_with_index do |ary, i|
      page = Hash.new
      page['rp'] = ary
      page['mark'] = (mark[i] || [])
      note['page'].push(page)
    end

    need_pages = (note['qr'].size.to_f / QR_PER_PAGE).ceil
    note['page'][need_pages - 1] = nil if note['page'].size < need_pages

    note['page'].push(nil) if note['page'].size == 0

    return note

  end

  def edit_lines(name, amount)
    lines = name.scan(/.{1,#{PS_SIZE}}/)
    unless amount.empty?
      if lines.size == 0 or lines.last.size >= AMOUNT_POS
        lines.push('　' * AMOUNT_POS  + amount)
      else
        lines[-1] = lines.last + '　' * (AMOUNT_POS - lines.last.size) + amount
      end
    end
    return lines
  end

  def edit_draft_zai

    zai = Array.new

    return zai if @data['Rp'].nil?

    no = 0
    @data['Rp'].each do |rp|
      next if rp['Medication'].nil?
      med = Hash.new
      med['rp'] = Array.new
      med['mark'] = Array.new
      rp['Medication'].each do |m|
        unless m['Name'].nil?
          med['mark'][med['rp'].size] = m['Unchangeable_Flg']
          amount = to_num(m['Amount']).to_s
          count = amount.index('.') || amount.size
          count = 5 if count > 5
          amount = (' ' * (5 - count) + amount).zenkaku + m['Unit_Name'].to_s
          med['rp'].concat(edit_lines(m['Name'], amount))
        end
      end

      unless rp['Unit_Name'].nil?
        text_count = ''
        if rp['Count'].nil?
          text_count = "（　　　#{rp['Unit_Name'].to_s}）"
        else
          text_count = ("（%3d#{rp['Unit_Name'].to_s}）" % rp['Count'].to_i).zenkaku
        end
        if med['rp'].last.size + text_count.size > PS_SIZE
#          med['rp'].push("　" * (PS_SIZE - text_count.size) + text_count)
           med['rp'].push(text_count.rjust(PS_SIZE, "　"))
        else
#          med['rp'][-1] = (med['rp'].last + "　" * (PS_SIZE - ( med['rp'].last.size + text_count.size)) + text_count)
           med['rp'][-1] = med['rp'].last + text_count.rjust(PS_SIZE - med['rp'].last.size, "　")
        end
      end
      unless rp['Split_Total_Count'].nil?
        text_split_total_count = "（総投与日数　#{rp['Split_Total_Count'].to_i.to_s.zenkaku}日）"
        med['rp'].push(text_split_total_count.rjust(PS_SIZE, "　"))
      end
      no += 1
      med['rp'].map!{|value| "　　　" + value}
      med['rp'][0][0,3] = ('%2d）' % no).zenkaku
      zai.push(med)
    end
    
    return zai
    
  end

  def edit_draft_memo2
    memo2 = Array.new
    MEMO2_ROW_SIZE.times do |i|
      memo2.push(Array(@data['Memo2'])[i] || '')
    end
    return memo2
  end

  def edit_draft_memo
    memo = Array.new
    Array(@data['Memo']).each do |text|
      range_begin = 0
      while text[range_begin..-1].to_s != ''
        length = (memo.size >= MEMO_ROW_SIZE ? NOTE_COL_SIZE : MEMO_COL_SIZE)
        memo.push(text[range_begin, length])
        range_begin += length
      end
    end
    return memo
  end

  def edit_draft
    draft = Hash.new
    draft['memo2'] = edit_draft_memo2
    draft['memo'] = edit_draft_memo
    draft['zai'] = edit_draft_zai
    return draft
  end

  def export(pdf_file, form_data, opt)

    offset_set = opt[:pr_conf][:offset_settings]
    if offset_set
      offset_1 =  offset_set[RED_FILE.to_sym]
      offset_2 =  offset_set[RED_FILE2.to_sym]
    else
      offset_1 = nil
      offset_2 = nil
    end	

    forms = Array.new
    @temp_dir = Dir.mktmpdir
    @opt = opt

    form_data['Forms'].each do |hash|
      @data = hash['data']
      draft = edit_draft
      note = edit_note(draft)
      forms.concat(edit_form(note, offset_1, offset_2))
    end

    Monpe.export(forms, pdf_file)
    FileUtils.remove_entry_secure @temp_dir
  end
end


=begin
pdf_file = "shohosen.pdf"
form_data = <<EOF
{
  "Forms": [
    {
      "data": {
      }
    }
  ]
}
EOF
conf = { :api_user => 'ormaster',
         :api_key => 'ormaster',
         :api_server => 'localhost',
         :api_port => 8000 }
opt = Hash.new
opt[:conf] = conf
require 'json'
form = Shohosen.new
form.export(pdf_file, JSON.parse(form_data), opt)
=end
