# coding : utf-8

require 'monpe'
require_relative '../lib/orca-wareki'
require_relative '../lib/orca-string'

class Taiin_shomeisho

  LINE_ROW_SIZE  = 69
  DESCRIPTION_SIZE  = 38
  RED_FILE  = 'HCN02V03.red'

  def export(fname, form_data, opt)

    offset_set = opt[:pr_conf][:offset_settings]
    if offset_set
      offset_1 =  offset_set[RED_FILE.to_sym]
    else
      offset_1 = nil
    end

    forms = []

    form_data['Forms'].each do |hash|
      @data = hash['data']
      f = edit_page
      f.set_offset(*offset_1) if offset_1
      forms << f
    end

    forms << edit_page if forms.size == 0

    Monpe.export(forms, fname)
  end

  def edit_page
    red = File.join(File.dirname(File.expand_path(__FILE__)), RED_FILE)
    form = Monpe::Report.new(red)
    embed_data = form.get_embed_data

    set_head(embed_data)

    form.set_embed_data(embed_data)
    return form
  end

  def set_head(embed_data)
    patient = @data['Patient'] || {}
    hospital = @data['Hospital'] || {}
    doctor = @data['Doctor'] || {}

    embed_data['HOSPNAME'] = [hospital['Name']].join.to_s[0, 26]
    embed_data['HOSPADRS'] = [hospital['Address']].join.to_s[0, 26]
    embed_data['HOSPTEL'] = hospital['PhoneNumber']

    embed_data['DRNAME'] = doctor['Name'].to_s[0,20]

    embed_data['PTNAME'] = patient['Name'].to_s[0,24]
    embed_data['PTADRS'] = [patient['Address']].join.to_s[0, 34]
    embed_data['PTTEL'] = patient['PhoneNumber']
    embed_data['PTSEX[0]'] = '○' if patient['Sex'] == '1'
    embed_data['PTSEX[1]'] = '○' if patient['Sex'] == '2'

    unless patient['BirthDate'].to_s.strip.empty?
      ymd = Orca::Wareki.new(patient['BirthDate'])
      ['明治', '大正', '昭和', '平成'].each_with_index do |gengo, i|
        embed_data["PTBIRTHG[#{i}]"] = '○' if gengo == ymd.gengoN
      end
      embed_data['PTBIRTHYMD.PTBIRTHYY'] = ('%2d' % ymd.year_w).zenkaku
      embed_data['PTBIRTHYMD.PTBIRTHMM'] = ('%2d' % ymd.month).zenkaku
      embed_data['PTBIRTHYMD.PTBIRTHDD'] = ('%2d' % ymd.day).zenkaku
    end

    embed_data['PTNENREI'] = patient['Age'].to_i

    if ! ['9999' , '' ].include?(@data['Admission_Date'].to_s[0,4].strip)
      ymd = Orca::Wareki.new(@data['Admission_Date'])
      embed_data['NYUINYMD.NYUINGEN'] = ymd.gengoN
      embed_data['NYUINYMD.NYUINYY'] = ('%2d' % ymd.year_w).zenkaku
      embed_data['NYUINYMD.NYUINMM'] = ('%2d' % ymd.month).zenkaku
      embed_data['NYUINYMD.NYUINDD'] = ('%2d' % ymd.day).zenkaku
    end

    if ! ['9999' , '' ].include?(@data['Discharge_Date'].to_s[0,4].strip)
      ymd = Orca::Wareki.new(@data['Discharge_Date'])
      embed_data['TAIINYMD.TAIINGEN'] = ymd.gengoN
      embed_data['TAIINYMD.TAIINYY'] = ('%2d' % ymd.year_w).zenkaku
      embed_data['TAIINYMD.TAIINMM'] = ('%2d' % ymd.month).zenkaku
      embed_data['TAIINYMD.TAIINDD'] = ('%2d' % ymd.day).zenkaku
    end

    h_charge_t = @data['Hospital_Charge_And_Term'] || {}
    h_charge = h_charge_t['Hospital_Charge'] || []
    term = h_charge_t['Term'] || {}

    h_charge.each_with_index do |charge, i|
      embed_data["SYUBETU[#{i}]"] = charge['Name'].to_s[0,25]
      embed_data["NISSU[#{i}]"] = "#{charge['Days']}日間" unless charge['Days'].to_s.strip.empty?
    end

    embed_data['KIKAN'] = term['Days'] unless term['Days'].to_s.strip.empty?

    if ! ['9999' , '' ].include?(term['Start_Date'].to_s[0,4].strip)
      ymd = Orca::Wareki.new(term['Start_Date'])
      embed_data['KIKANSTRYMD.KIKANSTRGEN'] = ymd.gengoN
      embed_data['KIKANSTRYMD.KIKANSTRYY'] = ('%2d' % ymd.year_w).zenkaku
      embed_data['KIKANSTRYMD.KIKANSTRMM'] = ('%2d' % ymd.month).zenkaku
      embed_data['KIKANSTRYMD.KIKANSTRDD'] = ('%2d' % ymd.day).zenkaku
    end

    if ! ['9999' , '' ].include?(term['End_Date'].to_s[0,4].strip)
      ymd = Orca::Wareki.new(term['End_Date'])
      embed_data['KIKANENDYMD.KIKANENDGEN'] = ymd.gengoN
      embed_data['KIKANENDYMD.KIKANENDYY'] = ('%2d' % ymd.year_w).zenkaku
      embed_data['KIKANENDYMD.KIKANENDMM'] = ('%2d' % ymd.month).zenkaku
      embed_data['KIKANENDYMD.KIKANENDDD'] = ('%2d' % ymd.day).zenkaku
    end

    embed_data['YOBI'] = "上記他#{h_charge_t['Other_Days']}日有" unless h_charge_t['Other_Days'].to_i == 0
 
    g_h_charge = @data['General_Hospital_Charge'] || {}

    embed_data['TKIKAN'] = g_h_charge['Days'] unless g_h_charge['Days'].to_s.strip.empty?

    if ! ['9999' , '' ].include?(g_h_charge['Base_Date'].to_s[0,4].strip)
      ymd = Orca::Wareki.new(g_h_charge['Base_Date'])
      embed_data['TKIKANYMD.TKIKANGEN'] = ymd.gengoN
      embed_data['TKIKANYMD.TKIKANYY'] = ('%2d' % ymd.year_w).zenkaku
      embed_data['TKIKANYMD.TKIKANMM'] = ('%2d' % ymd.month).zenkaku
      embed_data['TKIKANYMD.TKIKANDD'] = ('%2d' % ymd.day).zenkaku
    end

    if @data['Disease_Name']
      embed_data['BYOMEI.BYOMEI-S[0]'] = @data['Disease_Name'].to_s[0,38]
      embed_data['BYOMEI.BYOMEI-S[1]'] = @data['Disease_Name'].to_s[38,38]
    end
  end

end

=begin
pdf_file = "taiin_shomeisho.pdf"

form_data = <<EOF
{
  "Forms":
  [
    {
      "data":
      {
      }
    }
  ]
}
EOF

conf = { :api_user => 'ormaster',
         :api_key => 'ormaster',
         :api_server => 'localhost',
         :api_port => 8000 }
opt = Hash.new
opt[:conf] = conf
require 'json'
form = Taiin_shomeisho.new
form.export(pdf_file, JSON.parse(form_data), opt)
=end
