# coding : utf-8

require 'monpe'
require_relative '../lib/orca-wareki'

class Yoyakuhyo

  LINE_ROW_SIZE  = 12
  RED_FILE  = 'HCM67.red'


  def wareki_n(ymd)
    ymd.to_s.strip.empty? ? '' : Orca::Wareki.wareki_n(ymd)
  end

  def export(fname, form_data, opt)

    offset_set = opt[:pr_conf][:offset_settings]
    if offset_set
      offset_1 =  offset_set[RED_FILE.to_sym]
    else
      offset_1 = nil
    end

    forms = []

    form_data['Forms'].each do |hash|
      @data = hash['data']
      ary = edit_ary
      ary.each_slice(LINE_ROW_SIZE).each.with_index(1) do |ary, page_no|
        f = edit_page(ary, page_no)
        f.set_offset(*offset_1) if offset_1
        forms << f
      end
    end

    forms << edit_page if forms.size == 0

    Monpe.export(forms, fname)
  end

  def edit_ary
    ary = Array.new

    Array(@data['Appointment_List']).reject{|h| h['Date'].nil?}.each do |hash|
      ary.push({}) unless ary.empty?

      note = 'Note'
      if hash[note].nil?
        ary.push(hash)
      else
        hash[note].size.times do |i|
          if i == 0 then
            ary.push(Hash[hash.map{|k,v| k == note ?  [k ,v[i]] : [k, v]}])
          else
            ary.push(Hash[note, hash[note][i]]) unless hash[note][i].to_s.empty?
          end
        end
      end
    end

    return ary
  end

  def edit_page(ary = nil, page_no = nil)
    red = File.join(File.dirname(File.expand_path(__FILE__)), RED_FILE)
    form = Monpe::Report.new(red)
    embed_data = form.get_embed_data

    unless @data.nil?
      set_head(embed_data, page_no)
      Array(ary).each_with_index do |hash, i|
        set_list(embed_data, hash, i)
      end
    end

    form.set_embed_data(embed_data)
    return form
  end

  def set_head(embed_data, page_no)
    patient = @data['Patient'] || {}
    hospital = @data['Hospital'] || {}
    address = hospital['Address'] || []
    name = hospital['Name'] || []

    embed_data['PRTYMD'] = wareki_n(@data['IssuedDate'])

    embed_data['PAGE'] = page_no.to_s
    embed_data['PTNUM'] = patient['ID']
    embed_data['NAME'] = patient['Name'].to_s[0,15]
    embed_data['MSG'] = @data['Memo']

    if address[1].to_s.empty?
      embed_data['HOSPADRS'] = address[0].to_s[0,40]
    else
      embed_data['HOSPADRS'] = "#{address[0].to_s[0,20].ljust(20,"　")}#{address[1].to_s[0,20]}"
    end
    if name[1].to_s.empty?
      embed_data['HOSPNAME'] = name[0].to_s[0,40]
    else
      embed_data['HOSPNAME'] = "#{name[0].to_s[0,20].ljust(20,"　")}#{name[1].to_s[0,20]}"
    end
    embed_data['HOSPTEL'] = hospital['PhoneNumber']
  end

  def set_list(embed_data, hash, i)
    doctor = hash['Doctor'] || {}

    embed_data["NAIYO-TBL[#{i}].YYKYMD"] = wareki_n(hash['Date'])
    embed_data["NAIYO-TBL[#{i}].YYKTIME"] = hash['Time']
    embed_data["NAIYO-TBL[#{i}].SRYKA"] = hash['Department_Name'].to_s[0,5]
    embed_data["NAIYO-TBL[#{i}].DRNAME"] = doctor['Name'].to_s[0,9]
    embed_data["NAIYO-TBL[#{i}].BIKOH"] = hash['Note'].to_s[0,24]
  end

end

=begin
pdf_file = "yoyakuhyo.pdf"

form_data = <<EOF
{
  "Forms": [
    {
      "data": {
      }
    }
  ]
}
EOF

conf = { :api_user => 'ormaster',
         :api_key => 'ormaster',
         :api_server => 'localhost',
         :api_port => 8000 }
opt = Hash.new
opt[:conf] = conf
require 'json'
form = Yoyakuhyo.new
form.export(pdf_file, JSON.parse(form_data), opt)
=end
