# coding : utf-8

require 'monpe'
require_relative '../lib/orca-wareki'

class Yoyakukanjalist

  LINE_ROW_SIZE  = 33
  RED_FILE  = 'HCM15.red'

  def wareki_n(ymd)
    ymd.to_s.strip.empty? ? '' : Orca::Wareki.wareki_n(ymd)
  end

  def export(fname, form_data, opt)

    offset_set = opt[:pr_conf][:offset_settings]
    if offset_set
      offset_1 =  offset_set[RED_FILE.to_sym]
    else
      offset_1 = nil
    end

    forms = []

    form_data['Forms'].each do |hash|
      @data = hash['data']
      ary = edit_ary
      ary.each_slice(LINE_ROW_SIZE).each.with_index(1) do |ary, page_no|
        f = edit_page(ary, page_no)
        f.set_offset(*offset_1) if offset_1
        forms << f
      end
    end

    forms << edit_page if forms.size == 0

    Monpe.export(forms, fname)
  end

  def edit_ary
    ary = Array.new
    Array(@data['Appointment_Information']).each_with_index do |info, day|
      Array(info['Appointment_List']).reject{|h| h['Time'].nil?}.each_with_index do |hash, i|
        ary[i] ||= Array.new
        ary[i][day] = hash
      end
    end
    return ary
  end

  def edit_page(ary = nil, page_no = nil)
    red = File.join(File.dirname(File.expand_path(__FILE__)), RED_FILE)
    form = Monpe::Report.new(red)
    embed_data = form.get_embed_data

    unless ary.nil?
      set_head(embed_data, page_no)
      Array(ary).each_with_index do |hash, i|
        set_list(embed_data, hash, i)
      end
    end

    form.set_embed_data(embed_data)
    return form
  end

  def set_head(embed_data, page_no)
    medical = @data['Medical_Information'] || {}
    doctor = @data['Doctor'] || {}

    embed_data['PAGE'] = page_no.to_s
    embed_data['DRCD'] = doctor['Name'][0,10]
    embed_data['HAKYMD'] = wareki_n(@data['IssuedDate'])
    embed_data['TITLE'] = '予約患者一覧'
    embed_data['TITLE2'] = medical['Name']

    Array(@data['Appointment_Information']).each_with_index do |info, day|
      unless info['Date'].nil?
        ymd = Orca::Wareki.new(info['Date'])
        embed_data["YYKYMD[#{day}]"] = ymd.wareki_n + '（' + ymd.wday_n + '）'
      end
    end
  end

  def set_list(embed_data, hash, i)
    hash.each_with_index do |info, day|
      next if info.nil?
      medical = info['Medical_Information'] || {}
      patient = info['Patient'] || {}
      embed_data["TBL[#{day}].SRYNAIYO[#{i}]"] = medical['Name'][0,5]
      embed_data["TBL[#{day}].YYKTIME[#{i}]"] = info['Time']
      embed_data["TBL[#{day}].NAME[#{i}]"] = patient['TemporaryName'][0,7]
    end
  end

end

=begin
pdf_file = "yoyakukanjalist.pdf"

form_data = <<EOF
{
  "Forms": [
    {
      "data": {
      }
    }
  ]
}
EOF

conf = { :api_user => 'ormaster',
         :api_key => 'ormaster',
         :api_server => 'localhost',
         :api_port => 8000 }
opt = Hash.new
opt[:conf] = conf
require 'json'
form = Yoyakukanjalist.new
form.export(pdf_file, JSON.parse(form_data), opt)
=end
