# coding:utf-8
require 'pp'
require 'logger'
require 'yaml'
require 'fileutils'
require 'push_exchanger'

class Server_print

  include PushExchanger::Util

  def initialize(conf)
    Encoding.default_external = Encoding::UTF_8
    @conf = conf
    @logger = conf[:logger]
    filename = File.join(File.dirname(File.expand_path(__FILE__)),"server_print.yml")
    begin
      @sp_conf = YAML.load(File.read(filename))
      @pr_map = {}
      @sp_conf[:printer_settings].each do |k,v|
        n = 1
        if /(.*)#(\d+)$/ =~ k
          k = $1.to_sym
          n = $2.to_i
        end
        @pr_map[k] = {
          :printer => v,
          :num     => n
        }
      end
    rescue Exception
      @logger.info("server_print plugin init failure: could not read config")
    end
  end

  def get_events
    if @sp_conf
      %w|server_print_ready|
    else
      []
    end
  end

  def server_print_ready(data)
    @logger.info("server_print_ready start")
    id       = data["body"]["id"]
    dir      = @sp_conf[:report_dir]   || "/var/tmp/push-exchanger/server_print"
    filename = File.join(dir,Time.now.strftime("%Y%m%d%H%M%S-#{id[0,5]}.pdf"))
    limit    = @sp_conf[:report_limit] || 100
    _printer = data["body"]["printer"]

    pr = @pr_map[_printer.to_sym] || @pr_map[:default]
    unless pr
      raise "cannot found printer setting:#{_printer}"
    end
    printer = pr[:printer]
    num     = pr[:num]

    begin
      res = http_get(@conf,"/blobapi/#{id}")
      unless res.code == "200"
        raise "blobapi failure #{res.code}"
      end
      file_logger(dir,filename,limit) do
        File.open(filename,"wb") do |io| io.write res.body end
      end
      print(filename,printer,num,@sp_conf[:print_command])
    rescue Exception => ex
      STDERR.puts ex.backtrace
      @logger.info ex
      raise ex
    end
    @logger.info("server_print_ready end")
  end

end
