# coding:utf-8
require 'pp'
require 'logger'
require 'yaml'
require 'fileutils'
require 'push_exchanger'

class Up_sample1

  include PushExchanger::Util

  def initialize(conf)
    Encoding.default_external = Encoding::UTF_8
    @conf = conf
    @logger = conf[:logger]
    filename = File.join(File.dirname(File.expand_path(__FILE__)),"up_sample1.yml")
    begin
      @up_conf = YAML.load(File.read(filename))
      FileUtils.mkdir_p(@up_conf[:outdir])
    rescue Exception
      @logger.info("up_sample1 plugin init failure: could not read config")
    end
  end

  def get_events
    if @up_conf
      %w|accept| 
    else
      []
    end
  end

  def accept(data)
    @logger.info("up_sample1 accept start")

    # PUSH通知から患者番号を取り出す
    pid = data["body"]["Patient_ID"]

    begin
      # 患者基本情報取得APIの実行
      res = http_get(@conf,"/api01rv2/patientgetv2?id=#{pid}") 

      # APIレスポンスをファイルに書き出す
      outfile = File.join(@up_conf[:outdir],"#{pid}.xml")
      File.open(outfile,"wt") do |io| io.print res.body end 
    rescue Exception => ex
      STDERR.puts ex.backtrace
      @logger.info ex
      raise ex
    end
    @logger.info("up_sample1 accept end")
  end

end
